/*
   normal -- function to return a normal distribution of variables about
             a mean of zero, with a variance of one.  Adapted from Numerical
             Recipes, section 7.2.  Uses as input a uniform distribution
             over the interval (0,1).

             Written by Paul Hardy, USC, July 1994.
*/
#include <math.h>

double normdist()
{
static int have1=0;
static double nextval;
register double v1,v2;		/* uniform variable #1 and #2         */
double rsq, nlrsq;		/* rsq=v1^2 + v2^2, lrsq is -log(rsq) */
register double factor;		/* for computing sin, cos             */
double retval;			/* return value                       */
double log(),sqrt();
double GFSR();		/* uniform random number over (0,1)   */

if (have1) {
   retval = nextval;
   have1 = 0;
 }
else {
   do {				/* get point inside unit circle except (0,0) */
      v1 = GFSR();
      v1 = v1 + v1 - 1.0;	/* -1 < v1 < 1 */
      v2 = GFSR();
      v2 = v2 + v2 - 1.0;	/* -1 < v2 < 1 */
      rsq = v1*v1 + v2*v2;
    } while (rsq <= 0 || rsq >= 1.0);
   nlrsq = -log(rsq);
   factor = sqrt((nlrsq + nlrsq) / rsq);
   nextval = v1 * factor;
   retval  = v2 * factor;
   have1 = 1;
 }
return(retval);
}

/*
   Routine for Fortran compatibility.
*/
double normdist_() {
double normdist();
return(normdist());
}
