/* i_mfitD -- interface for matrix fit difference scoring */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 2.0 releast date: November 1996 -- adapted for database use.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>

#include "seqaln.h"

int main(argc,argv)
	int argc;
	char **argv;
{
  struct SEQALN_SEQUENCE *seq1,*seq2; /* First & second sequences to be compared. */
  struct SEQALN_CONSTANTS  *scoring;	/* Parameters related to scoring operations. */
  struct SEQALN_RESULTS  *results;	 /* Computation results. */
  struct SEQALN_IO *io;

  int i,j;
  int seq1status;
  extern char *newmem();

  seq1 = (struct SEQALN_SEQUENCE *)newmem(1, sizeof(struct SEQALN_SEQUENCE));
  seq2 = (struct SEQALN_SEQUENCE *)newmem(1, sizeof(struct SEQALN_SEQUENCE));
  scoring = (struct SEQALN_CONSTANTS *)newmem(1, sizeof(struct SEQALN_CONSTANTS));
  results = (struct SEQALN_RESULTS *)newmem(1, sizeof(struct SEQALN_RESULTS));
  io = (struct SEQALN_IO *)newmem(1, sizeof(struct SEQALN_IO));

  io->STDIN = stdin;
  io->STDOUT = stdout;
  io->STDERR = stderr;

  if (argc<7)
    {
      (void)fprintf(io->STDOUT,"Shortcut:\n\n   %s file1 file2 ", argv[0]);
      (void)fprintf(io->STDOUT,"match mismatch ");
      (void)fprintf(io->STDOUT,"alpha beta ");
      (void)fprintf(io->STDOUT,"[flags]\n\n");
    }
  scoring->ptype = FIT;
  scoring->stype = MATRIX;
  scoring->mtype = DISTANCE;
  scoring->btype = FULL;
  scoring->memuse = SQUARE;
  parseargs(seq1,seq2,scoring,results,io,argc,argv); /* Parse program arguments */
  results_init(seq1,seq2,scoring,results,io); /* Initialize results data struct */

  /*
    We have all the parameter information.  Now proceed w/computation.
  */
  seq1status = seq1->len;
  while (seq1status > 0) {  /* Stop at 0-length seq, or EOF (seq1status==-1) */
    fitD(seq1,seq2,scoring,results,io);
    if (scoring->memuse == LINEAR) {
      rpt_align(seq1,seq2,scoring,results,io);
    }
    else {
      do {
	for (i=0; i<=scoring->rptboth; i++) {
	  trace(seq1,seq2,scoring,results,io);
	  if (scoring->rpttrace) {
	    print_trace(scoring,results,io);
	  }
	  if (scoring->rptscore || scoring->rptalign) {
	    gen_align(seq1,seq2,scoring,results,io); /* Generate alignment */
	    rpt_align(seq1,seq2,scoring,results,io); /* Print alignment */
	  }
	  if (scoring->rptboth) {
	    scoring->traceupper = !scoring->traceupper;
	    (void)fprintf(io->STDOUT,"\n");
	  }
	}
	if (--results->naligns > 0 && results->bestS > scoring->cutoff &&
	    results->bestSj > 0) {
	  fddeclump(seq1,seq2,scoring,results,io);	/* Undo prev. align */
	  if (scoring->rptmat) {
	    rpt_scores(seq1,seq2,scoring,results,io);
	  }
	}
      } while (results->naligns > 0 && results->bestS > scoring->cutoff &&
	       results->bestSj > 0);
    }
    seq1status = load_seq(seq1,scoring,io);
    if (seq1status > 0) {
      results_init(seq1,seq2,scoring,results,io);
    }
  }  /* End of while seq1status . . . */
  
  exit(0);
}
