/*
   i_mpvlocalS.c -- interface for pvalue local matrix similarity scoring.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 2.0 releast date: November 1996 -- adapted for database use.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>

#include "seqaln.h"

int main(argc, argv)
     int argc;
     char *argv[];
{
  struct SEQALN_SEQUENCE *seq1,*seq2; /* 1st & 2nd sequences to be compared */
  struct SEQALN_CONSTANTS *scoring; /* Parameters related to scoring operation */
  struct SEQALN_RESULTS *results;	/* Computation results. */
  struct SEQALN_IO *io;

  int i,j;
  int seq1status;
  double nextpval();
  extern char *newmem();

  seq1 = (struct SEQALN_SEQUENCE *)newmem(1, sizeof(struct SEQALN_SEQUENCE));
  seq2 = seq1;   /* Unique sequence for self-repeats */
  scoring = (struct SEQALN_CONSTANTS *)newmem(1, sizeof(struct SEQALN_CONSTANTS));
  results = (struct SEQALN_RESULTS *)newmem(1, sizeof(struct SEQALN_RESULTS));
  results->pvalue = (struct SEQALN_PVALUE *)calloc(1,sizeof(struct SEQALN_PVALUE));
  io = (struct SEQALN_IO *)newmem(1, sizeof(struct SEQALN_IO));

  io->STDIN = stdin;
  io->STDOUT = stdout;
  io->STDERR = stderr;

  if (argc<6)
    {
      (void)fprintf(io->STDOUT, "Shortcut: %s ", argv[0]);
      (void)fprintf(io->STDOUT, "file matrix offset alpha beta ");
      (void)fprintf(io->STDOUT, "[flags]\n\n");
    }

  scoring->ptype = LOCAL;
  scoring->stype = MATRIX;
  scoring->mtype = SIMILARITY;
  scoring->btype = SELF;
  scoring->memuse = SQUARE;
  scoring->stats = PVALUE;
  scoring->allscores = 0;	/* just use middle 80% */
  parseargs(seq1,seq2,scoring,results,io,argc,argv); /* Parse program arguments */
  fclose(seq1->fptr);

  if (scoring->randfile != (char *)NULL) {
     readGFSR(scoring->randfile);  /* Initialize GFSR state table */
     }
  if (scoring->p == 0.0 || scoring->gamma == 0.0) { /* unknown; compute */
     pvsrlocalS(seq1,seq2,scoring,results,io);
     (void)fprintf(io->STDOUT,
           "P-value parameters: gamma = %g, p = %g, coeff = %g\n",
      results->pvalue->gamma, results->pvalue->p, results->pvalue->coeff);
     }
  else {
     results_init(seq1,seq2,scoring,results,io);
     results->pvalue->gamma  = scoring->gamma;
     results->pvalue->p = scoring->p;
     (void)fprintf(io->STDOUT,
           "P-value parameters: gamma = %g, p = %g\n",
        results->pvalue->gamma, results->pvalue->p);
     (void)fprintf(io->STDOUT,
           "P(Score <= x) ~ e**(%g * %d * %d * %g**x\n",
       -results->pvalue->gamma, seq1->len, seq2->len, results->pvalue->p);
     }

  /*
     Now load the sequence back into its arrays.
  */
  seq1->fptr = (FILE *)NULL;  /* To force file re-open */
  load_seq(seq1);
  /*
     We have all the parameter information.  Now proceed w/computation.
  */
  seq1status = seq1->len;
  while (seq1status > 0) {  /* Stop at 0-length seq, or EOF (seq1status==-1) */
    srlocalS(seq1,seq2,scoring,results,io);
    if (scoring->memuse == LINEAR) {
      nextpval(seq1,seq2,scoring,results,io);
      (void)fprintf(io->STDOUT,"\nP(Score[%d] > %d) ~ %g\n",
		    results->pvalue->npvals, results->bestS, results->pvalue->pvalue);
    }
    else {
      do {
	nextpval(seq1,seq2,scoring,results,io);
	(void)fprintf(io->STDOUT,"\nP(Score[%d] > %d) ~ %g\n",
		      results->pvalue->npvals, results->bestS, results->pvalue->pvalue);
	for (i=0; i<=scoring->rptboth; i++) {
	  trace(seq1,seq2,scoring,results,io);  /* Perform multiple tracebacks */
	  if (scoring->rpttrace) {
	    print_trace(scoring,results,io);
	  }
	  if (scoring->rptalign || scoring->rptscore) {
	    gen_align(seq1,seq2,scoring,results,io); /* Generate alignment */
	    rpt_align(seq1,seq2,scoring,results,io); /* Print alignment */
	  }
	  if (scoring->rptboth) {
	    scoring->traceupper = !scoring->traceupper;
	    (void)fprintf(io->STDOUT,"\n");
	  }
	}
	if (--results->naligns > 0 && results->bestS > scoring->cutoff) {
	  lsdeclump(seq1,seq2,scoring,results,io);  /* Undo previous alignment */
	  if (scoring->rptmat) {
	    rpt_scores(seq1,seq2,scoring,results,io);
	  }
	}
      } while (results->naligns > 0 && results->bestS > scoring->cutoff);
    }
    seq1status = load_seq(seq1,scoring,io);
    if (seq1status > 0) {
      results_init(seq1,seq2,scoring,results,io);
    }
  }  /* End of while seq1status . . . */

  if (scoring->randfile != (char *)NULL) {
     writeGFSR(scoring->randfile);  /* Save GFSR state table */
     }

  exit(0);
}
