/* i_pfitS.c -- interface for profile fit similarity scoring */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 2.0 releast date: November 1996 -- adapted for database use.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>

#include "seqaln.h"

int main(argc,argv)
	int argc;
	char **argv;
{
  struct SEQALN_PROFILE  *pro; /* The profile information. */
  struct SEQALN_SEQUENCE *seq; /* Sequence to be compared with profile. */
  struct SEQALN_CONSTANTS  *scoring; /* Scoring constants. */
  struct SEQALN_RESULTS  *results;   /* Computation results. */
  struct SEQALN_IO *io;

  int i,j;
  int seq1status;
  extern char *newmem();

  pro = (struct SEQALN_PROFILE *)newmem(1, sizeof(struct SEQALN_PROFILE));
  seq = (struct SEQALN_SEQUENCE *)newmem(1, sizeof(struct SEQALN_SEQUENCE));
  scoring = (struct SEQALN_CONSTANTS *)newmem(1, sizeof(struct SEQALN_CONSTANTS));
  results = (struct SEQALN_RESULTS *)newmem(1, sizeof(struct SEQALN_RESULTS));
  io = (struct SEQALN_IO *)newmem(1, sizeof(struct SEQALN_IO));

  io->STDIN = stdin;
  io->STDOUT = stdout;
  io->STDERR = stderr;

  if (argc<5)
    {
      (void)fprintf(io->STDOUT,"Shortcut:\n\n   %s profile seqfile ", argv[0]);
      (void)fprintf(io->STDOUT,"alpha beta ");
      (void)fprintf(io->STDOUT,"[flags]\n\n");
    }
  scoring->ptype = FIT;
  scoring->stype = PROFILE;
  scoring->mtype = SIMILARITY;
  scoring->btype = FULL;
  scoring->memuse = SQUARE;
  /* Parse program arguments */
  parseargs((struct SEQALN_SEQUENCE *)pro,seq,scoring,results,io,argc,argv);
  results_init(pro,seq,scoring,results,io); /* Initialize results data struct */

  /*
    We have all the parameter information.  Now proceed w/computation.
  */
  seq1status = seq->len;
  while (seq1status > 0) {
    pfitS(pro,seq,scoring,results,io);
    if (scoring->memuse == LINEAR) {
      rpt_align(pro,seq,scoring,results,io);	/* print score */
    }
    else {
      do {
	for (i=0; i<=scoring->rptboth; i++) {
	  ptrace(pro,seq,scoring,results,io);
	  if (scoring->rpttrace) {
	    print_trace(scoring,results,io);
	  }
	  if (scoring->rptscore || scoring->rptalign) {
	    gen_align(pro,seq,scoring,results,io); /* Generate the alignment */
	    rpt_align(pro,seq,scoring,results,io); /* Print the alignment */
	  }
	  if (scoring->rptboth) {
	    scoring->traceupper = !scoring->traceupper;
	    (void)fprintf(io->STDOUT,"\n");
	  }
	}
	if (--results->naligns > 0 && results->bestS > scoring->cutoff &&
	    results->bestSj > 0) {
	  pfsdeclump(pro,seq,scoring,results,io);	/* Undo prev. align */
	  if (scoring->rptmat) {
	    rpt_scores(pro,seq,scoring,results,io);
	  }
	}
      } while (results->naligns > 0 && results->bestS > scoring->cutoff &&
	       results->bestSj > 0);
    }
    seq1status = load_seq(seq,scoring,io);
    if (seq1status > 0) {
      results_init(seq,pro,scoring,results,io);
    }
  }  /* End of while seq1status . . . */

  exit(0);
}
