/*
   insert.c -- program to insert files when signaled by a line beginning with
               ".so ".

               Performs a Boyer-Moore string search for "\n.so " through the
               input file.

               Paul Hardy, USC, October 1993.
*/

#include <stdio.h>
#include <string.h>

#define BUFFSIZE (64*1024)

main() {

char *pattern="\n.so ";		/* The pattern to search for; has no repeats */
int  skip[256];			/* The amount to skip when each char is seen */
int  i,j;			/* Loop variables                            */
int  togo;			/* Characters left to go in input search     */
int  nextout;			/* Position of next character to write       */
int  plen;			/* Length of pattern                         */
char srcbuff[BUFFSIZE];		/* Input buffer, also written to output      */
int  inlen;			/* Number of characters in input buffer      */
char tmpname[256];		/* Name of inserted file                     */
int  tmplen;			/* Length of data in tmpbuff                 */
char tmpbuff[BUFFSIZE];		/* Buffer for reading/writing inserted files */
FILE *instream,*outstream;	/* Input and output file streams             */
FILE *tmpstream;		/* Temp stream, for reading inserted files   */

plen = strlen(pattern);

/*
   By default, skip the entire pattern over any letter encountered (this is
   corrected for characters actually in the pattern below).
*/
for (i=0; i<256; i++) {
   skip[i] = plen;
   }
/*
   Skip just enough that if a letter in the pattern is found, we align the
   two to perform a search (the Boyer-Moore string search algorithm).
*/
for (i=plen-1; i>=0; i--) {
   skip [ pattern [ i ] ] -= i+1;
   }

/*
   Now read the input buffer (stdin for now) and write to the output buffer
   (stdout for now).  Open and insert into the output stream any files to be
   included.
*/
nextout = plen;

while ((inlen = fread(&srcbuff[plen],1,BUFFSIZE-plen,stdin)) > 0) {
   for (i = plen; i < (inlen+plen); ) {
      if (skip[srcbuff[i]] != 0) { /* Boyer-Moore string search iteration. */
	 i += skip[srcbuff[i]];
         }
      else {			/* Might have found the pattern. */
	 if (strncmp(&srcbuff[i-plen+1],pattern,plen) == 0) { /* Found it. */
	    /*
	       Print everything up to the pattern.
	    */
	    fwrite(&srcbuff[nextout],1,i-nextout-plen+1,stdout);
	    fputc('\n',stdout);	/* Separate files with a newline */
	    while (isspace(srcbuff[++i]));
	    j=0;
	    while (!isspace(srcbuff[i]))
	       tmpname[j++] = srcbuff[i++];
	    tmpname[j]='\0';
	    if ((tmpstream = fopen(tmpname,"r")) == NULL) {
	       fprintf(stderr,"Error: can't insert file %s\n",tmpname);
	       }
	    else {		/* Insert file into the output stream */
	       while ((tmplen = fread(tmpbuff,1,BUFFSIZE,tmpstream)) > 0) {
		  fwrite(tmpbuff,1,tmplen,stdout);
		  }
	       fclose(tmpstream);
	       }
            while (srcbuff[i++] != '\n');  /* Skip past end of insert line */
	    nextout = i;
	    }
	 else {
	    i += plen;		/* Move search beyond this character. */
	    }
         }
      }
   if (nextout < (inlen - plen)) { /* Write to plen chrs of end of buffer. */
      fwrite(&srcbuff[nextout],1,i-nextout+1,stdout);
      }
   nextout=0;
   }
}

