/* fddeclump.c -- undo the effects of a fit distance alignment. */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include "seqaln.h"

fddeclump(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
  int i, j;
  int infty;  /* infinity -- i.e., a lower score than possible */
  int E;
  int *F;

  extern int gap(),max(),max3();

  F = results->F;
  infty = gap(seq1->len + seq2->len, scoring);
  for (j=1; j<=seq2->len; j++)
     F[j] = infty;

  /*
     Set substitution (diagonal) flags by walking through traceback once.
  */
  markdiag(seq1,seq2,scoring,results,io);
  results->aligned[results->bestSi][results->bestSj] = 1; /* don't end here */

  /* iterative S for LINEAR weighting */
  if (scoring->alpha == scoring->beta) {/* don't use E, F matrices */
     for (i=1; i<=seq1->len; i++) {
        for (j=1; j<=seq2->len; j++) {
	   if (results->aligned[i][j]) { /* don't try substitution */
	      results->S[i][j] = min(results->S[i-1][j] + scoring->alpha,
		 		    results->S[i][j-1] + scoring->alpha);
	    }
	   else {
	      results->S[i][j]=min3(results->S[i-1][j-1] +
		       scoring->deltafn(seq1->code[i],seq2->code[j], scoring),
				    results->S[i-1][j] + scoring->alpha,
				    results->S[i][j-1] + scoring->alpha);
	   }
	   if (scoring->Debug) {
	     (void)fprintf(io->STDOUT,
			   "\ndelta(%c,%c) = %5d;  E = %5d;  F[%d] = %5d;  ",
			   seq1->seq[i], seq2->seq[j],
		   scoring->deltafn(seq1->code[i], seq2->code[j],scoring),
			   E, j, F[j]);
	     (void)fprintf(io->STDOUT,
			  "S[%2d][%2d] = %5d\n", i, j, results->S[i][j]);
	   }
       }
     }
  }
  else {
     for (i=1; i<=seq1->len; i++) {
        E = infty;
	for (j=1; j<=seq2->len; j++) {
	   /* E[i][j] == min over k {S[i][j-k]+gap(k,scoring)} */
	   E = min(results->S[i][j-1] + scoring->alpha,
		   E + scoring->beta);

	   /* F[i][j] == min over k {S[i-k][j]+gap(k,scoring)} */
	   F[j] = min(results->S[i-1][j] + scoring->alpha,
		      F[j] + scoring->beta);
	  
	   if (results->aligned[i][j]) { /* don't try substitution */
	       results->S[i][j] = min(E, F[j]);
	     }
	   else {
	      results->S[i][j]=min3(results->S[i-1][j-1] +
		       scoring->deltafn(seq1->code[i],seq2->code[j], scoring),
				    E, F[j]);
	   }
	   if (scoring->Debug) {
	     (void)fprintf(io->STDOUT,
			   "\ndelta(%c,%c) = %5d;  E = %5d;  F[%d] = %5d;  ",
			   seq1->seq[i], seq2->seq[j],
		   scoring->deltafn(seq1->code[i], seq2->code[j],scoring),
			   E, j, F[j]);
	     (void)fprintf(io->STDOUT,
			  "S[%2d][%2d] = %5d\n", i, j, results->S[i][j]);
	   }
	}
     }
  }


  /*
     Get the new best score, from the bottom row.
  */
  results->bestSi = i = 0;
  results->bestSj = j = seq2->len;
  results->bestS = results->S[i][j];
  for (i=1; i<=seq1->len; i++) {
     if (results->S[i][j] < results->bestS && !results->aligned[i][j]) {
        results->bestS = results->S[i][j];
        results->bestSi = i;
        }
     }

  return;
}
