/* fitS.c -- fit similarity. */
/*
   Author: Paul Hardy, U.S.C., December 1995, 1997.

   Version 2.0 (transpose fit matrix): March 1997.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include "seqaln.h"

fitS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
  int i, j;
  int E;
  int neginfty;

  int *fabove;
  int *sdiag;
  int *sabove;
  int *sleft;
  int *scurrent;

  extern int gap(),max(),max3();

neginfty = gap(seq1->len * seq2->len, scoring);

  /* make sure this is reset if not 1st time through */
  for (j=1; j<=seq2->len; j++)  results->F[j] = neginfty;

  sdiag = results->S[0];
  sabove = sdiag + 1;
  sleft = results->S[1];
  scurrent = sleft + 1;

  if (scoring->rptmat) {
     rpt_row(seq1,seq2,scoring,results,io,0); /* set i=0 to initialize */
     }

  /* Initialize so we find the best score along the rightmost column */
results->bestSi = 0;
results->bestSj = seq2->len;
results->bestS = neginfty;

  /* recursive S for LINEAR weighting */
  if (scoring->alpha == scoring->beta) { /* don't use E, F */
    for (i=1; i<=seq1->len; i++) {
      for (j=1; j<=seq2->len; j++, sdiag++, sabove++, sleft++, scurrent++) {
	 *scurrent = max3(*sdiag +
		  scoring->deltafn(seq1->code[i], seq2->code[j], scoring),
				 *sabove + scoring->alpha,
				 *sleft  + scoring->alpha);
      }
      if (scoring->rptmat) {
         rpt_row(seq1,seq2,scoring,results,io,i);
         }
      if (*sleft > results->bestS) { /* new best score in final column */
	 results->bestSi = i;
	 results->bestS = *sleft;
         }
      if (scoring->memuse == LINEAR) {
         sdiag = results->S[i & 1];
         sleft = results->S[1 - (i & 1)];
         }
      else {
         sdiag = results->S[i];
         sleft = results->S[i+1];
         }
      sabove = sdiag + 1;
      scurrent = sleft + 1;
    }
  }
  else {
     for (i=1; i<=seq1->len; i++) {
        E = neginfty;		/* don't use the first time through */
	fabove = &results->F[1];
	for (j=1;
             j<=seq2->len;
             j++, sdiag++, sabove++, sleft++, scurrent++, fabove++) {
	   /* E[i][j] == max over k {S[i][j-k]+gap(k,scoring)} */
	   E = max(*sleft + scoring->alpha, E + scoring->beta);

	   /* F[i][j] == max over k {S[i-k][j]+gap(k,scoring)} */
	   *fabove = max(*sabove + scoring->alpha,
			 *fabove + scoring->beta);
	  
	   *scurrent=max3(*sdiag +
		    scoring->deltafn(seq1->code[i], seq2->code[j], scoring),
				 E, *fabove);
	}
      if (scoring->rptmat) {
         rpt_row(seq1,seq2,scoring,results,io,i);
         }
      if (*sleft > results->bestS) { /* new best score in final column */
	 results->bestSi = i;
	 results->bestS = *sleft;
         }
      if (scoring->memuse == LINEAR) {
         sdiag = results->S[i & 1];
         sleft = results->S[1 - (i & 1)];
         }
      else {
         sdiag = results->S[i];
         sleft = results->S[i+1];
         }
      sabove = sdiag + 1;
      scurrent = sleft + 1;
     }
  }

  return;
}
