/* gen_align.c -- routines to generate alignment strings */
/*
   Author: Paul Hardy, February 1997.
*/

#include <stdio.h>
#include "seqaln.h"

void gen_align(seq1,seq2,scoring,results,io)
   struct SEQALN_SEQUENCE *seq1,*seq2;
   struct SEQALN_CONSTANTS *scoring;
   struct SEQALN_RESULTS   *results;
   struct SEQALN_IO *io;
{
void tandem_gen();
void norm_gen();

if (scoring->btype == TANDEM) {
   tandem_gen(seq1,seq2,scoring,results,io);
   }
else {
   norm_gen(seq1,seq2,scoring,results,io);
   }

return;
}


/*
   norm_gen() -- routine to generate alignment strings for
                   normal alignments.
*/
void norm_gen(seq1,seq2,scoring,results,io)
   struct SEQALN_SEQUENCE *seq1,*seq2;
   struct SEQALN_CONSTANTS *scoring;
   struct SEQALN_RESULTS   *results;
   struct SEQALN_IO *io;
{
int i,j,k;          /* positions in strings we build */
int tracei, tracej; /* positions in traceback        */
int newalen;        /* length of this new alignment  */
extern char *newmem();
SEQALN_TRACE *ttrace = results->trace;

if (scoring->memuse == LINEAR) return;
if (results->trace == (SEQALN_TRACE *)NULL) return;

newalen=0;  /* First character */
if (scoring->Debug) {
   printf("newalen = %d\n", newalen);
   }
ttrace = results->trace;

/*
   Determine starting position.
*/
if (ttrace != (SEQALN_TRACE *)NULL) {
   if (scoring->ptype == LOCAL ||
         (scoring->ptype == OVER && results->trace->i != 0))
      tracei = results->start1 = results->trace->i - 1;
   else
      tracei = results->start1 = 0;
   if (scoring->ptype == LOCAL || 
      ((scoring->ptype == FIT || scoring->ptype == OVER)
          && results->trace->j != 0))
      tracej = results->start2 = results->trace->j - 1;
   else
      tracej = results->start2 = 0;
   }

if (scoring->Debug) {
   (void)fprintf(io->STDOUT,"%d  %d  %d\n",tracei,tracej,newalen);
   }

while (ttrace != (SEQALN_TRACE *)NULL) {
   if (tracei < ttrace->i) {
      if (tracej < ttrace->j) {  /* Two letters are aligned */
         newalen++;
         }
      else {  /* Indel */
         newalen += ttrace->i - tracei;
         }
      }
   else if (tracej < ttrace->j) {  /* Indel */
      newalen += ttrace->j - tracej;
      }
   tracei = ttrace->i;
   tracej = ttrace->j;
   if (scoring->Debug) {
      (void)fprintf(io->STDOUT,"%d  %d  %d\n",tracei,tracej,newalen);
      }
   if (scoring->Debug) {
      printf("newalen = %d\n", newalen);
      }
   ttrace = ttrace->next;
   }

if (results->align0 != (char *)NULL)
   free(results->align0);
if (results->align1 != (char *)NULL)
   free(results->align1);
if (results->align2 != (char *)NULL)
   free(results->align2);

results->align_len = newalen;

results->align0 = newmem(results->align_len+1, sizeof(char));
results->align1 = newmem(results->align_len+1, sizeof(char));
results->align2 = newmem(results->align_len+1, sizeof(char));
i = 0;  /* Positions in results->align{1,2,0} */

/*
   Determine the starting location in both sequences if traceback isn't NULL.
*/
ttrace = results->trace;
tracei = results->start1;  /* Computed above */
tracej = results->start2;  /* Computed above */
/*
   Now walk through the traceback, building both alignment strings as we go.
*/
while (ttrace != (SEQALN_TRACE *)NULL) {
   if (scoring->Debug) {
      (void)fprintf(io->STDOUT,"(%d,%d)\t",ttrace->i,ttrace->j);
      }
   if (tracei < ttrace->i && tracej < ttrace->j) {  /* Substitution */
      if (scoring->Debug) {
         (void)fprintf(io->STDOUT,"Align(%d,%d) %d\n",tracei,tracej,i);
         }
      results->align2[i] = seq2->seq[++tracej];
      results->align1[i] =
         scoring->stype == PROFILE ?
            results->align2[i] : seq1->seq[++tracei];
      if (results->align1[i] == results->align2[i]) {  /* Match */
         if (scoring->showmmatch) {
            results->align0[i] = ' ';
            }
         else {
            results->align0[i] = '|';
            }
         }
      else if (scoring->mtype == SIMILARITY &&
               scoring->stype != PROFILE) {
         if (scoring->deltafn(seq1->code[ttrace->i],
                              seq2->code[ttrace->j],
                              scoring) > 0) {
            results->align0[i] = ':';  /* conservative match */
            }
         else if (scoring->showmmatch) {
            results->align0[i] = '*';
            }
         else {
            results->align0[i] = ' ';
            }
         }
      else {  /* Distance scoring */
         results->align0[i] = ' ';
         }
      if (scoring->Debug) {
         (void)fprintf(io->STDOUT,"[%c,%c]\n", seq1->seq[ttrace->i], seq2->seq[ttrace->j]);
         }
      tracei = ttrace->i;
      tracej = ttrace->j;
      i++;
      }
   else {  /* Indel */
      while (tracei < ttrace->i) {  /* Insertion in first sequence */
         tracei++;
         if (scoring->Debug) {
            (void)fprintf(io->STDOUT,"Indel2(%d,%d) %d\n",tracei,tracej,i);
            }
         results->align0[i] = ' ';
         results->align1[i] =
            scoring->stype == PROFILE ? 'X' : seq1->seq[tracei];
         results->align2[i] = '-';
         i++;
         }
      while (tracej < ttrace->j) {  /* Insertion in second sequence */
         tracej++;
         if (scoring->Debug) {
            (void)fprintf(io->STDOUT,"Indel1(%d,%d) %d\n",tracei,tracej,i);
            }
         results->align0[i] = ' ';
         results->align1[i] = '-';
         results->align2[i] = seq2->seq[tracej];
         i++;
         }
      }
   ttrace = ttrace->next;
   }
/*
   Null-terminate the strings -- we're done.
*/
results->align0[i] = results->align1[i] = results->align2[i] = '\0';

return;
}

/*
   Define how we detect alignment events in a tandem matrix.
*/
#define tandalign(i0,j0,i1,j1) (i1 == tandnexti(i0) && j1 == tandnextj(j0) &&  \
              results->S[i1][j1] == results->S[i0][j0] +           \
              scoring->deltafn(seq1->code[i1],seq2->code[j1+1],scoring))
#define tanddelseq1(i0,j0,i1,j1) (i0 == i1 && \
               (j0 < j1 || (j0 == jmax && j1 == 0 && !align(i0,j0,i1,j1))))
#define tanddelseq2(i0,j0,i1,j1) (i0  < i1 && j0 == j1)
#define tandprevi(i0) (i0 == 0 ?    0 : i0 - 1)
#define tandprevj(j0) (j0 == 0 ? jmax : j0 - 1)
#define tandnexti(i0) (i0 + 1)
#define tandnextj(j0) (j0 == jmax ? 0 : j0 + 1)

/*
   tandem_gen() -- routine to generate alignment strings for
                     tandem alignments.
*/
void tandem_gen(seq1,seq2,scoring,results,io)
   struct SEQALN_SEQUENCE *seq1,*seq2;
   struct SEQALN_CONSTANTS *scoring;
   struct SEQALN_RESULTS   *results;
   struct SEQALN_IO *io;
{
int i,j,k;          /* positions in strings we build */
int jmax;           /* maximum j subscript           */
int lasti, lastj;   /* previous i,j in traceback     */
int tracei, tracej; /* positions in traceback        */
int newalen;        /* length of this new alignment  */
int nwraps;         /* number of matrix wraparounds  */
extern char *newmem();
SEQALN_TRACE *ttrace = results->trace;

if (scoring->memuse == LINEAR) return;
if (results->trace == (SEQALN_TRACE *)NULL) return;

jmax = seq2->len - 1;
nwraps=0;           /* no wraparounds yet */
newalen=0;          /* First character    */
if (scoring->Debug) {
   printf("newalen = %d\n", newalen);
   }
ttrace = results->trace;

/*
   Determine starting position for tandem local alignment.
*/
if (ttrace != (SEQALN_TRACE *)NULL) {
   tracei = results->start1 = results->trace->i;
   tracej = results->start2 = results->trace->j + 1;
   }

if (scoring->Debug) {
   (void)fprintf(io->STDOUT,"%d  %d  %d\n",tracei,tracej,newalen);
   }

while (ttrace != (SEQALN_TRACE *)NULL) {
   if (tandalign(tracei,ttrace->i,tracej,ttrace->j)) {  /* Two letters are aligned */
      newalen++;
      }
   else {  /* Indel */
      newalen += ttrace->i - tracei;
      newalen += ttrace->j - tracej;
      }
   tracei = ttrace->i;
   tracej = ttrace->j;
   if (scoring->Debug) {
      (void)fprintf(io->STDOUT,"%d  %d  %d\n",tracei,tracej,newalen);
      }
   if (scoring->Debug) {
      printf("newalen = %d\n", newalen);
      }
   ttrace = ttrace->next;
   }

if (results->align0 != (char *)NULL)
   free(results->align0);
if (results->align1 != (char *)NULL)
   free(results->align1);
if (results->align2 != (char *)NULL)
   free(results->align2);

results->align_len = newalen;

results->align0 = newmem(results->align_len+1, sizeof(char));
results->align1 = newmem(results->align_len+1, sizeof(char));
results->align2 = newmem(results->align_len+1, sizeof(char));
i = 0;  /* Positions in results->align{1,2,0} */

/*
   Determine the starting location in both sequences if traceback isn't NULL.
*/
ttrace = results->trace;
tracei = results->start1;  /* Computed above */
tracej = results->start2;  /* Computed above */
/*
   Now walk through the traceback, building both alignment strings as we go.
*/
while (ttrace != (SEQALN_TRACE *)NULL) {
   if (scoring->Debug) {
      (void)fprintf(io->STDOUT,"(%d,%d)\t",ttrace->i,ttrace->j);
      }
   if (tandalign(tracei,ttrace->i,tracej,ttrace->j)) {  /* Substitution */
      if (scoring->Debug) {
         (void)fprintf(io->STDOUT,"Align(%d,%d) %d\n",tracei,tracej,i);
         }
      results->align2[i] = seq2->seq[tandnextj(tracej)];
      results->align1[i] =
         scoring->stype == PROFILE ?
            results->align2[i] : seq1->seq[tandnexti(tracei)];
      if (results->align1[i] == results->align2[i]) {  /* Match */
         if (scoring->showmmatch) {
            results->align0[i] = ' ';
            }
         else {
            results->align0[i] = '|';
            }
         }
      else if (scoring->mtype == SIMILARITY &&
               scoring->stype != PROFILE) {
         if (scoring->deltafn(seq1->code[ttrace->i],
                              seq2->code[ttrace->j],
                              scoring) > 0) {
            results->align0[i] = ':';  /* conservative match */
            }
         else if (scoring->showmmatch) {
            results->align0[i] = '*';
            }
         else {
            results->align0[i] = ' ';
            }
         }
      else {  /* Distance scoring */
         results->align0[i] = ' ';
         }
      if (scoring->Debug) {
         (void)fprintf(io->STDOUT,"[%c,%c]\n", seq1->seq[ttrace->i], seq2->seq[ttrace->j]);
         }
      tracei = ttrace->i;
      tracej = ttrace->j;
      i = tandnexti(i);
      }
   else {  /* Indel */
      while (tracei < ttrace->i) {  /* Insertion in first sequence */
         tracei = tandnexti(tracei);
         if (scoring->Debug) {
            (void)fprintf(io->STDOUT,"Indel2(%d,%d) %d\n",tracei,tracej,i);
            }
         results->align0[i] = ' ';
         results->align1[i] =
            scoring->stype == PROFILE ? 'X' : seq1->seq[tracei];
         results->align2[i] = '-';
         i = tandnexti(i);
         }
      while (tracej < ttrace->j) {  /* Insertion in second sequence */
         tracej = tandnextj(tracej);
         if (scoring->Debug) {
            (void)fprintf(io->STDOUT,"Indel1(%d,%d) %d\n",tracei,tracej,i);
            }
         results->align0[i] = ' ';
         results->align1[i] = '-';
         results->align2[i] = seq2->seq[tracej];
         i = tandnexti(i);
         }
      }
   ttrace = ttrace->next;
   }
/*
   Null-terminate the strings -- we're done.
*/
results->align0[i] = results->align1[i] = results->align2[i] = '\0';

return;
}
