/*
   gettop.c -- module to find the top score in a scoring matrix.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include <malloc.h>
#include "seqaln.h"

static int init=1;

gettop(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1, *seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
if (scoring->mtype == SIMILARITY)
   getmaxS(seq1,seq2,scoring,results,io);
else
   getminS(seq1,seq2,scoring,results,io);

return;
}

/*
   Precomputes the max score for a row/column.
*/
initmaxS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
int i,j,top,total;
int gaplen;
int limit;
char *malloc();
/*
   Find what the top score is for all letter combinations.
*/
topscore(seq1,seq2,scoring,results,io);

limit = min(seq1->len, seq2->len);

for (i=0, total = 0; i <= limit; i++, total += results->maxentry) {
   results->best[i] = total;
 }

/*
   Now decrease score as we encounter forced deletions after min. seq. len.
*/
limit = max(seq1->len, seq2->len);
total += scoring->alpha;

while (i <= limit) {
   results->best[i++] = total;
   total += scoring->beta;
   }

return;
}

/*
   Find the minimum score in a matrix.  Does not use results->aligned[][].
*/
getminS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
int *intp;
int i,j;
int bestS, bestSi, bestSj;

bestS = 0;
bestSi = 0;
bestSj = 0;

for (i=1; i<=seq1->len; i++) {
   for (j=1; j<=seq2->len; j++) {
      if (results->S[i][j] < bestS) {
         bestS = results->S[i][j];
         bestSi = i;
         bestSj = j;
         }
      }
   }

results->bestS  = bestS;
results->bestSi = bestSi;
results->bestSj = bestSj;

return;
}

/*
   Find the maximum score in a matrix, for local scoring.
*/
getmaxS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
int *intp;
int i,j;
int bestS, bestSi, bestSj;

if (scoring->naligns > 1 && results->aligned == (char **)NULL) {
   (void)fprintf(io->STDERR,
           "Error: results->aligned[][] not initialized.\n\n");
   exit(-1);
 }

bestS = 0;
bestSi = 0;
bestSj = 0;

for (i=1; i<=seq1->len; i++) {
   for (j=1; j<=seq2->len; j++) {
      if (results->S[i][j] > bestS &&
          (!results->aligned || !results->aligned[i][j])) {
         bestS = results->S[i][j];
         bestSi = i;
         bestSj = j;
         }
      }
   }

results->bestS  = bestS;
results->bestSi = bestSi;
results->bestSj = bestSj;

return;
}

/*
   Find the maximum score in a self-repeat matrix, for local self repeats.
   The matrix structure is lower-triangular.
*/
srgetmaxS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
int *intp;
int i,j;
int bestS, bestSi, bestSj;

if (scoring->naligns > 1 && results->aligned == (char **)NULL) {
   (void)fprintf(io->STDERR,
             "Error: results->aligned[][] not initialized.\n\n");
   exit(-1);
 }

if (init) {		/* in case programmer forgot, or doesn't know */
   initmaxS(seq1,seq2,scoring,results,io);
 }

bestS = -1; /* we'll at least encounter a score of zero */

for (i=seq1->len; i>0 && results->best[i] >= bestS; i--) {
   for (j = i-1; j >= 0; j--) {
      if (results->S[i][j] > bestS &&
	  (results->aligned && results->aligned[i][j] == 0))
         { bestS = results->S[i][j]; bestSi = i; bestSj = j; }
    }
 }

results->bestS  = bestS;
results->bestSi = bestSi;
results->bestSj = bestSj;

return;
}
