/* globalD.c -- global distance. */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include "seqaln.h"

globalD(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
  int i, j;
  int E;
  int infty;

  int *fabove;
  int *sdiag;
  int *sabove;
  int *sleft;
  int *scurrent;

  extern int gap(),min(),min3();

infty = gap(seq1->len * seq2->len, scoring);

  /* make sure this is reset if not 1st time through */
  for (j=1; j<=seq2->len; j++)  results->F[j] = infty;

  results->S[0][0] = 0;
  results->S[1][0] = scoring->alpha;

  sdiag = results->S[0];
  sabove = sdiag + 1;
  sleft = results->S[1];
  scurrent = sleft + 1;

  if (scoring->rptmat) {
     rpt_row(seq1,seq2,scoring,results,io,0); /* set i=0 to initialize */
     }

  /* recursive S for LINEAR weighting */
  if (scoring->alpha == scoring->beta) { /* don't use E, F */
    for (i=1; i<=seq1->len; i++) {
      for (j=1; j<=seq2->len; j++, sdiag++, sabove++, sleft++, scurrent++) {
	 *scurrent = min3(*sdiag +
		  scoring->deltafn(seq1->code[i], seq2->code[j], scoring),
				 *sabove + scoring->alpha,
				 *sleft  + scoring->alpha);
      }
      if (scoring->rptmat) {
         rpt_row(seq1,seq2,scoring,results,io,i);
         }
      if (scoring->memuse == LINEAR) {
         sdiag = results->S[i & 1];
         sleft = results->S[1 - (i & 1)];
	 *sleft = *sdiag + scoring->beta; /* initial gap score */
         }
      else {
         sdiag = results->S[i];
         sleft = results->S[i+1];
         }
      sabove = sdiag + 1;
      scurrent = sleft + 1;
    }
  }
  else {
     for (i=1; i<=seq1->len; i++) {
        E = infty;		/* don't use the first time through */
	fabove = &results->F[1];
	for (j=1;
             j<=seq2->len;
             j++, sdiag++, sabove++, sleft++, scurrent++, fabove++) {
	   /* E[i][j] == min over k {S[i][j-k]+gap(k,scoring)} */
	   E = min(*sleft + scoring->alpha, E + scoring->beta);

	   /* F[i][j] == min over k {S[i-k][j]+gap(k,scoring)} */
	   *fabove = min(*sabove + scoring->alpha,
			 *fabove + scoring->beta);
	  
	   *scurrent=min3(*sdiag +
		    scoring->deltafn(seq1->code[i], seq2->code[j], scoring),
				 E, *fabove);
	}
      if (scoring->rptmat) {
         rpt_row(seq1,seq2,scoring,results,io,i);
         }
      if (scoring->memuse == LINEAR) {
         sdiag = results->S[i & 1];
         sleft = results->S[1 - (i & 1)];
	 *sleft = *sdiag + scoring->beta; /* initial gap score */
         }
      else {
         sdiag = results->S[i];
         sleft = results->S[i+1];
         }
      sabove = sdiag + 1;
      scurrent = sleft + 1;
     }
  }

  /*
     Now return the score in lower right-hand corner of the matrix.
  */
  results->bestSi = seq1->len;
  results->bestSj = seq2->len;
  if (scoring->memuse == LINEAR)
     results->bestS = results->S[results->bestSi & 1][results->bestSj];
  else
    results->bestS = results->S[results->bestSi][results->bestSj];

  return;
}
