/*
   linreg -- routine to perform linear regression.  Arguments are:

             n          number of elements in arrays x and y (input)
             x          array of x data (input)
             y          array of y data (where y = ax + b) (input)
             slope      value of 'a' in y = ax + b (output)
             intercept  value of 'b' in y = ax + b (output)
             coeff      reliability coefficient of linear regression (output)
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

linreg(n, x, y, slope, intercept, coeff)
	int n;
	double *x, *y;
	double *slope, *intercept, *coeff;
{
	int i;
	double thisx, thisy;
	double xbar=0.0, ybar=0.0;
	double xx=0.0, yy=0.0, xy=0.0;

	for (i=0; i<n; ++i) {
		xbar += x[i];
		ybar += y[i];
	}
	xbar /= (double)(n);
	ybar /= (double)(n);

	for (i=0; i<n; i++) {
		thisx = x[i] - xbar;
		thisy = y[i] - ybar;
		xy += thisx * thisy;
		xx += thisx * thisx;
		yy += thisy * thisy;
	}
	if (xx == 0 || yy == 0) {
	   *coeff = 0;
	   *slope = 0;
	   *intercept = 0;
	 }
	else {
	   *coeff = xy / sqrt(xx*yy);
	   *slope = xy / xx;
	   *intercept = ybar - *slope * xbar;
	 }

	return;
}

