/* code for loading a scoring matrix */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.2 (July 1996): updated to read BLAST-format scoring matrices
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include <stdlib.h>

#include "seqaln.h"

load_mat(fn,scoring,io)
     char *fn;
     struct SEQALN_CONSTANTS *scoring;
     struct SEQALN_IO *io;
{
  FILE *theMatrix;
  int i, j;
  char *mdir;
  char mfn[MAXPATH+1];
  char inbuf[MAXLINE];
  char *getenv(),*fgets();
  int  blast_format;     /* BLAST-format matrices                   */
  int  cols[32];         /* the order in which columns are assigned */
  int  row;              /* the current row number                  */
  int  col;              /* the current column number               */
  int  datalines;        /* number of lines of real data we read    */
  int  retval;           /* return value of routines being called   */
  
  theMatrix = fopen(fn, "r");
  if (theMatrix == NULL) {
     (void)sprintf(mfn, "%s.mat", fn);
     theMatrix=fopen(mfn, "r");
     if (theMatrix==NULL) {
        mdir=getenv("MATDIR");
        if (mdir==NULL) {
           (void)fprintf(io->STDERR, "bad matrix file\n");
           exit(2);
	}
        (void)sprintf(mfn, "%s/%s", mdir, fn);
        theMatrix=fopen(mfn, "r");
        if (theMatrix==NULL) {
           (void)sprintf(mfn, "%s/%s.mat", mdir, fn);
           theMatrix=fopen(mfn, "r");
           if (theMatrix==NULL) {
	      (void)fprintf(io->STDERR, "bad matrix file\n");
	      exit(2);
           }
	}
     }
  }
  /*
     The file is assumed to be in BLAST format until we read a non-blank line
     whose first character is not '#', a letter, or '*'.
  */
  blast_format = 1;                         /* Assume it's BLAST format      */
  cols[0] = -1;                             /* Signal the beginning          */
  datalines = 0;
  while (blast_format &&                    /* Until there are no more lines */
         fgets(inbuf, MAXLINE, theMatrix) != NULL) {
     for (i=0; isspace(inbuf[i]) &&
               inbuf[i] != '\n'; i++);      /* Skip to first non-blank       */
     if (inbuf[i] != '#') {                 /* If not a comment              */
        if (isalpha(inbuf[i]) ||
            inbuf[i] == '*') {              /* Matrix is BLAST-format        */
           datalines++;
           if (cols[0] < 0) {               /* First definition row          */
              col=0;
              while (inbuf[i] != '\n') {
                 while (!isalpha(inbuf[i]) &&
                        inbuf[i] != '*' &&
                        inbuf[i] != '\n') i++;
                 if (isalpha(inbuf[i]) || inbuf[i] == '*') {
                    cols[col] = asc2num(toupper(inbuf[i]));
                    col++;
                    i++;
                    }
                 }
              col = 0;
              }
           else {  /* We're reading a BLAST-format matrix */
              datalines++;
              row = asc2num(toupper(inbuf[i]));
              i++;
              while (inbuf[i] != '\n') {
                 while (!isdigit(inbuf[i]) &&
                        inbuf[i] != '-' &&
                        inbuf[i] != '\n') i++;
                 if (inbuf[i] != '\n') {
                    scoring->xmatrix[row][cols[col]] =
                       atoi(&inbuf[i]) + scoring->offset;
                    col++;
                    while (!isspace(inbuf[i])) i++; /* skip to next num */
                    }
                 }
              col = 0;
              }
           }
        else {  /* not a BLAST-format matrix; use loop below */
           blast_format = 0;
           }
        }
     }
  if (blast_format == 0) {  /* it wasn't BLAST format, but our format */
     rewind(theMatrix);
     retval = ! EOF;
     for (i=0; i<26 && retval != EOF; i++) {
        for (j=0; j<=i; j++) {
	   if ((retval = fscanf(theMatrix, "%d", &scoring->xmatrix[i][j]))
               != EOF) {
              datalines++;
	      scoring->xmatrix[i][j]+=scoring->offset;
	      scoring->xmatrix[j][i]=scoring->xmatrix[i][j];
              }
	   }
        }
     }
  (void)fclose(theMatrix);
  if (scoring->mtype == DISTANCE) { /* Change sign of scores if distance */
     for (i=0; i<=26; i++) {
        for (j=0; j<=26; j++) {
           scoring->xmatrix[i][j] = -scoring->xmatrix[i][j];
           }
        }
     }
  /*
     Print the matrix.
  */
  if (scoring->Debug) {
     printf("    A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z  *\n");
     for (i=0; i<=26; i++) {
        printf("%c ", i < 26 ? i + 'A' : '*');
        for (j=0; j<=26; j++)
           printf("%3d",scoring->xmatrix[i][j]);
        printf("\n");
        }
     }

  if (datalines == 0) {
     fprintf(io->STDERR,"\nNo data in substitution score matrix!\n\n");
     exit(1);
     }

  return(0);
}
