/* function to load the next sequence in file name into sequence struct;
   return value is length of sequence.  Opens the file the first time called.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 2.0  (PH): Set default frame start/end to cover whole sequence.
   Version 1.17 (PH): Now supports ONLY the FASTA format
                      Can read an entire database of sequences
                      First argument, "fn", moved into sequence->fname
   Version 1.16 (PH): Added to FASTA format to ignore lines starting with ';'.
   Version 1.1 (PH): February 1996: if first non-blank character is '>',
                     assume entire file is in FASTA format.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include <ctype.h>
#include "seqaln.h"

int load_seq(sequence, scoring, io)
     struct SEQALN_SEQUENCE *sequence;
     struct SEQALN_CONSTANTS *scoring;
     struct SEQALN_IO *io;
{
FILE *f;
int fasta_format;
int c;
char *a,*b;
int i,j;
char *newmem();
long ftell();

	if (sequence->fname == (char *)NULL) {
	   (void)fprintf(io->STDERR, "load_seq: sequence file not set.\n");
	   return(-1);
	   }
        if (strcmp(sequence->fname,"-") == 0) {  /* "-" denotes stdin */
           sequence->fptr = io->STDIN;
           }
	if (sequence->fptr == (FILE *)NULL) {
	   f=fopen(sequence->fname, "r");
	   if (f==NULL) {
	      (void)fprintf(io->STDERR, "bad file %s\n", sequence->fname);
	      sequence->len = 0;
	      return(-1);
	      }
	   sequence->fptr = f;
	   sequence->fposn = 0L;  /* start of file */
           }
	else {
	   f = sequence->fptr;
	   }
	/* Read from start of sequence */
	(void)fseek(f, sequence->fposn, SEEK_SET);
        i=0;
        while ((c = fgetc(f)) != '>' && c != EOF);
	while ((c = fgetc(f)) == '>'); /* Line can begin w/multiple >'s */
        /* Read the sequence name into the name array */
	for (i=0; i<MAXSEQNAME-1 && c != EOF && c != '\n'; i++) {
           if (c != '\r')
              sequence->name[i] = c;
           c = fgetc(f);
	   }
	sequence->name[i] = '\0';  /* nullify the newline */
	while ((c=fgetc(f)) == ';') { /* FASTA annotation line -- ignore */
	   while ((c=fgetc(f)) != '\n'); /* read to end of line */
	   }
        if (c != '\r')
	   ungetc(c,f);  /* part of seq -- put back to read below */

        /* Determine length of sequence. */
	sequence->fposn = ftell(f);  /* We'll return after memory alloc */
	for (j=0; (c=fgetc(f)) != EOF && c != '>' ; ) {
	   if (!isspace(c))
	      j++;
	   }
	sequence->len = j;
        /* Allocate memory to hold sequence if not enough alloc'd. */
	if (j > sequence->maxlen) {
	   sequence->resized = 1; /* mark size change, for results_init */
	   if (sequence->seq != (char *)NULL)
	      free(sequence->seq);
	   if (sequence->code != (char *)NULL)
	      free(sequence->code);
	   sequence->seq = newmem(j+2, sizeof(char));
	   sequence->code = newmem(j+2, sizeof(char));
	   sequence->maxlen = j;  /* We increased allocated sequence memory */
           }
        else {
           sequence->resized = 0;
           }
	a = sequence->seq;
	b = sequence->code;
	/* Position to start of sequence string */
	(void)fseek(f, sequence->fposn, SEEK_SET);
        /* Read sequence into allocated space. */
	for (a++, b++; (c=fgetc(f)) != EOF && c != '>' && c != ';' ; ) {
	   if (c && !isspace(c)) {
	      *a = toupper(c);
              *b = asc2num(*a);
              a++;
              b++;
	      }
	   }
	*a = '\0';
	while (c != EOF && c != '>') {
	   c = fgetc(f);
	   }
	if (c == '>') {
	   ungetc(c,f);
	   }
	sequence->fposn = ftell(f);  /* Start of new sequence */
        if (sequence->len <= 0) {
	   if (c != EOF) {
              fprintf(io->STDERR,"Null sequence encountered.\n\n");
	      }
           sequence->len = 0;
           sequence->seq = "";
           sequence->code = "";
           }
	if (j == 0 && c == EOF)  /* If no more data remains to be read */
	   j = -1;
        sequence->fstart = 1;    /* By default, start at seq beginning */
        sequence->fend   = j;    /* By default, stop at seq end        */
	return(j);        /* return length of sequence read; < 0 = EOF */
}
