/* lsdeclump.c -- undo the effects of a local similarity alignment. */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include "seqaln.h"

lsdeclump(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
  int i, j;
  int E;
  int *F;

  extern int gap(),max(),max3();

  F = results->F;
  for (j=1; j<=seq2->len; j++)
     F[j] = 0;

  /*
     Set substitution (diagonal) flags by walking through traceback once.
  */
  markdiag(seq1,seq2,scoring,results,io);

  /* iterative S for LINEAR weighting */
  if (scoring->alpha == scoring->beta) {/* Don't use E, F matrices */
     for (i=1; i<=seq1->len; i++) {
        E = 0;
	for (j=1; j<=seq2->len; j++) {
	   if (results->aligned[i][j]) { /* don't try substitution */
	      results->S[i][j] = max3(results->S[i-1][j] + scoring->alpha,
				      results->S[i][j-1] + scoring->alpha,
				      0);
	   }
	   else {
	      results->S[i][j]=max3(results->S[i-1][j-1] +
		       scoring->deltafn(seq1->code[i],seq2->code[j], scoring),
				    results->S[i-1][j] + scoring->alpha,
				    results->S[i][j-1] + scoring->alpha);
	      if (results->S[i][j] < 0) {
		 results->S[i][j] = 0;
	      }
	   }

	   if (scoring->Debug) {
              (void)fprintf(io->STDOUT,
			    "\ndelta(%c,%c) = %5d;  E = %5d;  F[%d] = %5d;  ",
			    seq1->seq[i], seq2->seq[j],
		       scoring->deltafn(seq1->code[i], seq2->code[j],scoring),
			    E, j, F[j]);
	      (void)fprintf(io->STDOUT,
			    "S[%2d][%2d] = %5d\n", i, j, results->S[i][j]);
	   }
	}
     }
  }
  else {
     for (i=1; i<=seq1->len; i++) {
        E = 0;
	for (j=1; j<=seq2->len; j++) {
	   /* E[i][j] == max over k {S[i][j-k]+gap(k,scoring)} */
	   E = max(results->S[i][j-1] + scoring->alpha,
		   E + scoring->beta);

	   /* F[i][j] == max over k {S[i-k][j]+gap(k,scoring)} */
	   F[j] = max(results->S[i-1][j] + scoring->alpha,
		      F[j] + scoring->beta);
	  
	   if (results->aligned[i][j]) { /* don't try substitution */
	      results->S[i][j] = max3(0, E, F[j]);
	   }
	   else {
	      results->S[i][j]=max3(results->S[i-1][j-1] +
		       scoring->deltafn(seq1->code[i],seq2->code[j], scoring),
				    E, F[j]);
	      if (results->S[i][j] < 0) {
		 results->S[i][j] = 0;
	      }
	   }

	   if (scoring->Debug) {
              (void)fprintf(io->STDOUT,
			    "\ndelta(%c,%c) = %5d;  E = %5d;  F[%d] = %5d;  ",
			    seq1->seq[i], seq2->seq[j],
		       scoring->deltafn(seq1->code[i], seq2->code[j],scoring),
			    E, j, F[j]);
	      (void)fprintf(io->STDOUT,
			    "S[%2d][%2d] = %5d\n", i, j, results->S[i][j]);
	   }
	}
     }
  }

  getmaxS(seq1,seq2,scoring,results,io);

  return;
}

