/*
   nextpval.c -- function to determine the next p-value for repeated aligns.
                 First time through, set results->pvalue->npvals = 0.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include "seqaln.h"
#include "mccaldon.h"

int nextpval(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS  *results;
struct SEQALN_IO *io;
{
  int i, j;
  static int k=0;			/* number of P-values computed */
  static double kfact=1.0;		/* kfact = k!                  */
  double lambda;
  double ifact;
  double sum;
  double exp(),pow(),floor();

  lambda = results->pvalue->gamma * seq1->len * seq2->len *
           pow(results->pvalue->p, (double)results->bestS);

  if (results->pvalue->npvals > 170) { /* can't compute > 170! on IEEE fp */
     results->pvalue->pvalue = 0.0;
     (void)fprintf(io->STDERR,
             "Warning: can't compute. %d! > 170! for P-value scores.\n",
	     results->pvalue->npvals);
     results->pvalue->npvals++;
     return;
   }

  if (k > results->pvalue->npvals) {
     k = 0;
     kfact = 1.0;
   }
  while (k < results->pvalue->npvals) {
     kfact *= ++k;
   }

  sum = 0.0;
  ifact = kfact;

  for (i=results->pvalue->npvals; i>0; i--) {
     sum   += pow(lambda, (double)i) / ifact;
     ifact  = floor(ifact / (double)i + 0.5);
   }
  sum += 1.0;			/* for lambda^0 / 0! = 1.0 */

  results->pvalue->npvals++;	/* so actually k = npvals - 1 */

  if (scoring->Debug)
    (void)fprintf(io->STDOUT,
           "\nP(score[%d] > %d) ~ 1 - %g * e^-%g\n",
	   results->pvalue->npvals,results->bestS, sum, lambda);
     
  sum = 1.0 - sum * exp(-lambda);
  if (sum < 1e-15)
     sum = 0.0;

  results->pvalue->pvalue = sum;

  return;
}
