/*
   parseflags.c -- routine to parse the flags provided to the main()
                   program module, as standard program options.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 2.0 (modified to prompt for arguments, xpose fit): PH, March 1997.
   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include "seqaln.h"

/* MAX_ARG_LEN is for reading arguments from prompts */
#ifndef MAX_ARG_LEN
#define MAX_ARG_LEN 2048
#endif

parseargs(seq1,seq2,scoring,results,io,argc,argv)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
int argc;
char *argv[];
{
int i,j;                         /* argv[i], for j...                       */
struct SEQALN_PROFILE *pro;      /* For loading a profile structure         */
char *trpattern;                 /* For repeating a pattern w/tandem repeat */
int  n;                          /* When we need number of elements         */
int prompts;                     /* Whether to prompt user for parameters   */
int prompterr;                   /* If parameter resulted in fatal error    */
char argline[MAX_ARG_LEN];       /* Arguments read in from user prompts     */
int delta(), mdelta(), pdelta();
extern int atoi();
extern double atof();
char *newmem(),*fgets();
/*   List of arguments for various programs (optional flags may follow args):
         globalS   db    seq2  match   mismatch  alpha  beta
        mglobalS   db    seq2  matrix  csub      alpha  beta
         globalD   db    seq2          mismatch  alpha  beta
        mglobalD   db    seq2  matrix  csub      alpha  beta

            fitS   db    seq2  match   mismatch  alpha  beta
           mfitS   db    seq2  matrix  csub      alpha  beta
           pfitS   db     pro                     alpha  beta
            fitD   db    seq2          mismatch  alpha  beta
           mfitD   db    seq2  matrix  csub      alpha  beta

           overS   db    seq2  match   mismatch  alpha  beta [+/-{123}]
          moverS   db    seq2  matrix  csub      alpha  beta [+/-{123}]

          localS   db    seq2  match   mismatch  alpha  beta
         mlocalS   db    seq2  matrix  csub      alpha  beta
        srlocalS   seq         match   mismatch  alpha  beta
       msrlocalS   seq         matrix  csub      alpha  beta
        trlocalS   seq   pat   match   mismatch  alpha
       mtrlocalS   seq   pat   matrix  csub      alpha
        pvlocalS   db    seq2  matrix  csub      alpha  beta
                                                        [dis2 [dis1]] [-g# -p#]
       mpvlocalS   db    seq2  matrix  csub      alpha  beta
                                                        [dis2 [dis1]] [-g# -p#]
      pvsrlocalS   seq         matrix  csub      alpha  beta
                                                        [dis2 [dis1]] [-g# -p#]
     mpvsrlocalS   seq         matrix  csub      alpha  beta
                                                        [dis2 [dis1]] [-g# -p#]
*/
/*
   Check for debugging output flag.
*/
for (j=1; argv[j]; j++) {
   if (argv[j][0] == '+' && argv[j][1] == 'D')
      scoring->Debug = 1;
   else if (argv[j][0] == '-' && argv[j][1] == 'D')
      scoring->Debug = 0;
   }
/*
   Determine if we need to prompt user for parameters.
*/
if (argc <= 1 || *argv[1] == '-' || *argv[1] == '+') {
   prompts = 1;
   }
else {
   prompts = 0;
   }

/*
  ARGUMENT: The first sequence
*/
i = 1;
if (prompts) {  /* Prompt user for information if necessary. */
   fprintf(io->STDOUT,"Name of first sequence/database file: ");
   fgets(argline,MAX_ARG_LEN,io->STDIN);
   argline[strlen(argline)-1] = '\0';  /* Change newline character to null */
   seq1->fname = newmem(strlen(argline)+1, sizeof(char));
   strcpy(seq1->fname, argline);
   }
else {
   seq1->fname = newmem(strlen(argv[i])+1, sizeof(char));
   strcpy(seq1->fname, argv[i++]);
}
if (load_seq(seq1, scoring, io) == 0) { /* load the first sequence or db */
   (void)fprintf(stderr,"Bad sequence file %s\n",seq1->fname);
   exit(1);
   }
/*
  ARGUMENT: The second sequence or a profile
*/
if (scoring->btype != SELF) { /* if there's a normal second sequence */
   if (scoring->stype == PROFILE) { /* load a profile as the first sequence */
      pro = (struct SEQALN_PROFILE *)seq2;  /* Convert back to profile */
     /*
        Scoring is passed along with pro to load
        scoring->palpha and scoring->pbeta
     */
     if (prompts) { /* Prompt user for file name for profile */
        fprintf(io->STDOUT,"Name of profile file: ",argline,MAX_ARG_LEN);
        }
     else {
        strcpy(argline,argv[i++]);
        }
     pro->fname = newmem(strlen(argline)+1, sizeof(char));
     strcpy(pro->fname,argline);
     if (load_pro(argline, pro, scoring, io) != 0) {
        (void)fprintf(stderr,"Bad profile file %s\n",argline);
        exit(1);
        }
     if (scoring->mtype == SIMILARITY) {
        pro->alpha = -pro->alpha;
        pro->beta  = -pro->beta;
        }
     seq1 = (struct SEQALN_SEQUENCE *)pro;
     }
   else {
      if (prompts) {  /* Prompt user for information if necessary. */
         fprintf(io->STDOUT,"Name of second sequence file: ");
         fgets(argline,MAX_ARG_LEN,io->STDIN);
         argline[strlen(argline)-1] = '\0';  /* Change '\n' to '\0' */
         seq2->fname = newmem(strlen(argline)+1, sizeof(char));
         strcpy(seq2->fname, argline);
         }
      else {
         seq2->fname = newmem(strlen(argv[i])+1, sizeof(char));
         strcpy(seq2->fname, argv[i++]);
         }
      if (load_seq(seq2, scoring, io) == 0) { /* load the second sequence */
         (void)fprintf(stderr,"Bad sequence file %s\n",seq2->fname);
         exit(1);
         }
    }
}
if (scoring->stype == MATRIX) {  /* matrix parameters: matrix name and csub */
   if (prompts) {  /* Prompt user for information if necessary. */
      fprintf(io->STDOUT,"Name of matrix file: ");
      fgets(argline,MAX_ARG_LEN,io->STDIN);
      argline[strlen(argline)-1] = '\0';  /* Change '\n' to '\0' */
      load_mat(argline, scoring, io);
      fprintf(io->STDOUT,"Limit for conservative substitutions: ");
      fscanf(io->STDIN,"%d",&scoring->csub);
      }
   else {
      load_mat(argv[i++], scoring, io);
      scoring->csub = atoi(argv[i++]);
      }
   }
/*
  ARGUMENT: match and/or mismatch score -- non-matrix software only.
*/
else {  /* non-matrix-scoring parameters */
   if (scoring->mtype == SIMILARITY) {
      if (scoring->stype != PROFILE) {
         if (prompts) {  /* Prompt user for match and mismatch. */
            fprintf(io->STDOUT,"Match score: ");
            fscanf(io->STDIN,"%d",&scoring->match);
            fprintf(io->STDOUT,"Mismatch score: ");
            fscanf(io->STDIN,"%d",&scoring->mismatch);
            scoring->mismatch = -scoring->mismatch;
            }
         else {
            scoring->match = atoi(argv[i++]);
            scoring->mismatch = -atoi(argv[i++]);
            }
         }
      }
   else {  /* Distance scoring if not similarity. */
      if (prompts) {  /* Prompt user for match (no mismatch with distance). */
         fprintf(io->STDOUT,"Match score: ");
         fscanf(io->STDIN,"%d",&scoring->match);
         }
      else {
         scoring->mismatch = 0;
         scoring->mismatch = atoi(argv[i++]);
         }
      }
   }  /*end of non-matrix parameters */
/*
  ARGUMENT: alpha and/or beta (beta not used for tandem repeats)
*/
if (scoring->mtype == SIMILARITY) {
   if (prompts) {  /* Prompt user for gap score parameters */
      fprintf(io->STDOUT,"Alpha, for gap score: ");
      fscanf(io->STDIN,"%d",&scoring->alpha);
      scoring->alpha = -scoring->alpha;
      if (scoring->btype != TANDEM) {
         fprintf(io->STDOUT,"Beta, for gap score: ");
         fscanf(io->STDIN,"%d",&scoring->beta);
         scoring->beta = -scoring->beta;
         }
      }
   else {
      scoring->alpha = -atoi(argv[i++]);
      if (scoring->btype != TANDEM) {
         scoring->beta = -atoi(argv[i++]);
         }
      }
   }
else {  /* Distance scoring */
   scoring->alpha = atoi(argv[i++]);
   if (scoring->btype != TANDEM) {
      scoring->beta = atoi(argv[i++]);
    }
 }
/*
  ARGUMENT: distribution files 1 and 2 for p-value computation
  [allocate pvalue structure first]
*/
if (scoring->stats == PVALUE) {
   if ((unsigned)results->pvalue == (unsigned)NULL) {
      results->pvalue =
         (struct SEQALN_PVALUE *)calloc(1,sizeof(struct SEQALN_PVALUE));
      }
   /* load distribution file(s) if specified */
   if (prompts) {  /* Prompt user for optional distribution files */
      fprintf(io->STDOUT,"Optional first file (database) distribution file: ");
      fgets(argline,MAX_ARG_LEN,io->STDIN);
      argline[strlen(argline)-1] = '\0';  /* Change '\n' to '\0' */
      if (strlen(argline) > 0) {  /* Let's keep going. */
         load_distrib(argline,results->pvalue->seq1dist,io);
         if (results->pvalue->seq1len < 0) {
	     (void)fprintf(stderr,"*** Error loading first distribution file.\n\n");
	      exit(1);
            }
         fprintf(io->STDOUT,"Optional second distribution file: ");
         fgets(argline,MAX_ARG_LEN,io->STDIN);
         argline[strlen(argline)-1] = '\0';  /* Change '\n' to '\0' */
         if (strlen(argline) > 0) {  /* Let's keep going. */
            load_distrib(argline,results->pvalue->seq2dist,io);
            if (results->pvalue->seq2len < 0) {
               (void)fprintf(stderr,"*** Error loading second distribution file.\n\n");
	        exit(1);
               }
            }
         }
      }
   else if (argv[i] && argv[i][0] != '-' && argv[i][0] != '+') { /* first file */
      results->pvalue->seq1len =
	load_distrib(argv[i++],results->pvalue->seq1dist,io);
      if (results->pvalue->seq1len < 0) {
	 (void)fprintf(stderr,"*** Error loading first distribution file.\n\n");
	 exit(1);
       }
      /* Got one distribution file; let's look for a second one. */
      if (argv[i] && argv[i][0] != '-' && argv[i][0] != '+') { /* second file */
         results->pvalue->seq2len =
	   load_distrib(argv[i++],results->pvalue->seq2dist,io);
         if (results->pvalue->seq2len < 0) {
	     (void)fprintf(stderr,"*** Error loading second distribution file.\n\n");
	     exit(1);
            }
         }
     }
 }
/*
   Default values for some constants inserted here.
*/
scoring->envelope = 0;		/* this isn't an alignment envelope    */
scoring->traceupper = 1;	/* trace upper indel direction first   */
scoring->flip = 0;		/* don't flip indel directions         */
scoring->rptalign = 1;		/* print the sequence alignment        */
scoring->rptscore = 1;		/* print the alignment score           */
scoring->rptmat = 0;		/* don't print the scoring matrix      */
scoring->rpttrace = 0;		/* don't print the traceback           */
scoring->rptboth = 0;           /* only report one traceback/alignment */
scoring->showmmatch = 0;	/* show alignment matches with `|'     */
io->STDIN = stdin;
io->STDOUT = stdout;
io->STDERR = stderr;
if (scoring->btype == TANDEM) {
   scoring->window = 1;		/* unless changed with -w# below       */
}
else {
  scoring->window = 0;
}
/*
   Assign match/mismatch scoring function.
*/
if (scoring->stype == MATRIX) {
   scoring->deltafn = mdelta;
 }
else if (scoring->stype == PROFILE) {
   scoring->deltafn = pdelta;
 }
else {
   scoring->deltafn = delta;
 }

/*
   Now parse any remaining flags, overriding defaults as requested.
*/
for (; i<argc; i++) {	/* loop through all args to look for flags */
    if (argv[i][0] == '-') {	/* it's a flag; parse it */
       switch(argv[i][1]) {
       case '|': scoring->showmmatch = 0; /* signal alignment matches */
                 break;
       case '*': scoring->showmmatch = 1; /* signal alignment mismatches */
                 break;
       case '1': scoring->overout = 2; /* don't report best score for 1st seq */
	         break;
       case '2': scoring->overout = 1; /* don't report best score for 2nd seq */
	         break;
       case 'A': scoring->rptalign = 0; /* don't report alignment */
	         break;
       case 'D': scoring->Debug = 0; /* turn off debugging output */
	         break;
       case 'E': io->STDERR = fopen(&argv[i][2],"w");
                 if (io->STDERR == NULL) {
                    (void)fprintf(stderr,
                          "*** Error: couldn't open %s for error output.\n",
                          &argv[i][2]);
                    io->STDERR = stderr;
                    }
                 break;
       case 'L': scoring->memuse = LINEAR;
                 break;
       case 'M': scoring->rptmat = 0; /* don't print scoring matrix */
	         break;
       case 'O': io->STDOUT = fopen(&argv[i][2],"w");
                 if (io->STDOUT == NULL) {
                    (void)fprintf(stderr,
                          "*** Error: couldn't open %s for standard output.\n",
                          &argv[i][2]);
                    io->STDOUT = stdout;
                    }
                 break;
       case 'P': scoring->rptptable = 0; /* don't print p-value prob. table */
                 break;
       case 'R': scoring->randfile = &argv[i][2]; /* GFSR seed file name */
                 break;
       case 'S': scoring->rptscore = 0;	/* don't print score */
	         break;
       case 'T': scoring->rpttrace = 0;	/* don't print traceback */
	         break;
       case 'V': scoring->verbose = 0;  /* don't print version number */
                 break;
       case 'W': scoring->width = atoi(&argv[i][2]); /* characters/line */
                 break;
       case 'a': scoring->allscores = 1; /* use all scores in p-val lin.reg. */
                 break;
       case 'b': scoring->rptboth = 1; /* report both upper & lower traces */
                 break;
       case 'c': scoring->cutoff = atoi(&argv[i][2]); /* cutoff score */
                 scoring->naligns = seq1->len * seq2->len; /* max # aligns */
	         break;
       case 'd': scoring->ndeclumps = atoi(&argv[i][2]);/* declumps/pval sim */
                 break;
       case 'e': scoring->envelope = 1;	/* trace an envelope */
	         break;
       case 'f': scoring->flip = 1; /* flip between upper & lower indel */
	         break;
       case 'g': scoring->gamma = atof(&argv[i][2]); /* for p-value aligns */
	         break;
       case 'l': scoring->traceupper = 0; /* prefer lower indel direction */
	         break;
       case 'n': scoring->naligns = atoi(&argv[i][2]); /* # of alignments */
                 scoring->cutoff = 0;  /* Stop when we reach a score of 0 */
	         break;
       case 'p': scoring->p = atof(&argv[i][2]); /* for p-value aligns */
	         break;
       case 'r': scoring->revcomp = 1; /* reverse-complement second sequence */
                 revcomp(seq2);
	         break;
       case 's': scoring->nsims = atoi(&argv[i][2]); /* # of seq simulations */
                 break;
       case 'u': scoring->traceupper = 1; /* prefer upper indel direction */
	         break;
       case 'w':
	 if (scoring->btype == TANDEM) {
	   scoring->window = atoi(&argv[i][2]); /* tr pattern copies */
	   if (scoring->window < 1) {
	     (void)fprintf(stderr,
			   "*** Error: need at least one copy of pattern.\n");
	     exit(1);
	   }
	   /* First repeat the sequence the requested number of times. */
	   if (scoring->window > 1) {/* if one copy, we've already got it */
	     n = seq2->len * scoring->window; /* chars in extended pattern */
	     trpattern = seq2->seq;
	     seq2->seq = (char *)newmem(n+2, sizeof(char));
	     for (j=0; j<n; j++) {
	       seq2->seq[j+1] = trpattern[j % seq2->len + 1];
	     }
	     free(trpattern);	/* original seq2->seq */
	     /* Now repeat the pattern code the requested number of times. */
	     trpattern = seq2->code;
	     seq2->code = (char *)newmem(n+2, sizeof(char));
	     for (j=0; j<n; j++) {
	       seq2->code[j+1] = trpattern[j % seq2->len + 1];
	     }
	     free(trpattern);	/* original seq2->code */
	     /* Update sequence length to accomodate number of repetitions. */
	     seq2->len = n;
	   }
	 } /* scoring->btype == TANDEM */
	 else {
	   (void)fprintf(stderr,
		    "*** Warning: -w# only supported for tandem repeats.\n");
	 }
       /* -x# -- nucleotide to protein translation */
       case 'x': seq2->xlated = 1;
                 xlate(seq2);
	         break;
       default: (void)fprintf(stderr,"*** Error: unknown flag %s\n\n",argv[i]);
	        break;
       }
     }
    else if (argv[i][0] == '+') {
       switch(argv[i][1]) {
       case 'A': scoring->rptalign = 1;	/* print alignment */
	         break;
       case 'D': scoring->Debug = 1; /* turn on debugging output */
	         break;
       case 'E': io->STDERR = fopen(&argv[i][2],"a");
                 if (io->STDERR == NULL) {
                    (void)fprintf(stderr,
                          "*** Error: couldn't append to %s for error output.\n",
                          &argv[i][2]);
                    io->STDERR = stderr;
                    }
                 break;
       case 'L': scoring->memuse = LINEAR;
                 break;
       case 'M': scoring->rptmat = 1; /* print scoring matrix */
	         break;
       case 'O': io->STDOUT = fopen(&argv[i][2],"a");
                 if (io->STDOUT == NULL) {
                    (void)fprintf(stderr,
                          "*** Error: couldn't append to %s for standard output.\n",
                          &argv[i][2]);
                    io->STDOUT = stdout;
                    }
                 break;
       case 'P': scoring->rptptable = 1; /* print p-value prob. table */
                 break;
       case 'S': scoring->rptscore = 1;	/* print alignment score */
                 break;
       case 'T': scoring->rpttrace = 1;	/* print traceback list */
	         break;
       case 'V': scoring->verbose = 1;  /* print version number */
                 rpt_version(argv[0],scoring);  /* print prog name w/version */
                 break;
       case '1': scoring->overout = 1; /* report best score for last row */
	         break;
       case '2': scoring->overout = 2; /* report best score for last column */
	         break;
       case '3': scoring->overout = 3; /* report scores from both endpoints */
	         break;
       default: (void)fprintf(stderr,"*** Error: unknown flag %s\n\n",argv[i]);
	        break;
       }
     }
  }

return;
}
