/* pfsdeclump.c -- profile fit similarity declump. */
/*
   Author: Paul Hardy, U.S.C., December 1995, 1997.

   Version 2.0 (transposed profile matrix): March 1997.
   Version 1.0 release date: December 1995.
*/

#include "seqaln.h"

pfsdeclump(seq,pro,scoring,results,io)
struct SEQALN_SEQUENCE *seq;
struct SEQALN_PROFILE  *pro;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS  *results;
struct SEQALN_IO *io;
{
  int i, j;
  int E, *F;
  int neginfty;

  extern char *newmem();
  extern int gap(),pgap(),max(),max3();

  F = results->F;
  for (j=1; j<=pro->len; j++) {
     F[j] = 0;
  }

  neginfty = gap(pro->len + seq->len, scoring);
  /*
    Set substitution (diagonal) flags by walking through traceback once.
    markdiag() doesn't read the pro data struct, so it's okay passing it
    here instead of a sequence.
  */
  markdiag(seq,pro,scoring,results,io);
  results->aligned[results->bestSi][results->bestSj] = 1; /* don't end here */

  /* recursive S for LINEAR weighting */
  if (scoring->palpha == scoring->pbeta) {/* Don't use E matrix */
     for (i=1; i<=seq->len; i++) {
        for (j=1; j<=pro->len; ++j) {
	   /* F[i][j] == max over k {S[i-k][j] + gap(k)} */
	   if (i>1)
	      F[j]=max(results->S[i-1][j] + scoring->alpha,
		       F[j] + scoring->beta);
	   else
	      F[j]=results->S[i-1][j] + scoring->alpha;

	   if (results->aligned[i][j]) { /* don't re-align */
	      results->S[i][j] = max(F[j], results->S[i][j-1]+pro->alpha);
	   }
	   else {
	      results->S[i][j]=max3(results->S[i-1][j-1] +
				    scoring->deltafn(j, seq->code[i], pro),
				    F[j], results->S[i][j-1]+scoring->palpha);
	   }
	}
     }
  }
  else {
     for (i=1; i<=seq->len; i++) {
        for (j=1; j<=pro->len; ++j) {
	   /* E[i][j] == max over k {S[i][j-k] + gap(k)} */
	   if (j>1)
	      E=max(results->S[i][j-1] + scoring->palpha,
		    E + scoring->pbeta);
	   else
	     E=results->S[i][j-1] + scoring->palpha;

	   /* F[i][j] == max over k {S[i-k][j] + gap(k)} */
	   if (i>1)
	      F[j]=max(results->S[i-1][j] + scoring->alpha,
		       F[j] + scoring->beta);
	   else
	      F[j]=results->S[i-1][j] + scoring->alpha;

	   if (results->aligned[i][j]) { /* don't re-align */
	      results->S[i][j] = max(F[j], E);
	   }
	   else {
	      results->S[i][j]=max3(results->S[i-1][j-1] +
				    scoring->deltafn(j, seq->code[i], pro),
				    F[j], E);
	   }
	}
     }
  }
  results->bestSi = i = 0;
  j = pro->len;
  results->bestS=results->S[i][j];
  for (i=1; i<=seq->len; i++) {
     if (results->S[i][j]>results->bestS && !results->aligned[i][j])	{
        results->bestS=results->S[i][j];
        results->bestSj=j;
     }
  }
  if (results->bestSi == 0)
     results->naligns = 0;

     return;
}
