/*
   pvlocalS.c -- module to compute p- and gamma-values.
*/
/*
   Author: Paul Hardy, U.S.C., August 1996.

   Version 1.17 release date: September 1996.
*/
#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include "seqaln.h"
#include "mccaldon.h"

pvsrlocalS(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS  *results;
struct SEQALN_IO *io;
{
  int i, j;
  int *p;
  char *charp;
  char *q;
  extern char *newmem();
  double slope, intercept, coeff, thegamma;
  int save1, save2;
  int collen, rowlen;
  int *sptr;
  char *malloc();

  save1=seq1->len;  /* Remember original sequence lengths. */

  if (scoring->ndeclumps > 0) {  /* need at least 900 * 900 to declump 300*  */
     seq1->len = max(900,seq1->len);
  }
  if (seq1->code != (char *)NULL) {
     free(seq1->code);
     }
  seq1->code = (char *)malloc((seq1->len + 1) * sizeof(char));

  /*
      Use uniform distributions of A,C,G,T if using non-matrix version and
      no distribution is specified, because we'll assume simple match-mismatch
      scoring with DNA/RNA.
  */
  if (seq1->freq == NULL)
     seq1->freq = (scoring->stype == NONMATRIX) ?
                  seqaln_dnadis : seqaln_mccaldon;
  results_init(seq1,seq2,scoring,results,io); /* allocate S[][], mainly */

  if (scoring->memuse == LINEAR && results->pvalue->ndeclumps > 0) {
     (void)fprintf(io->STDERR,"*** Error: can't declump in linear mode.\n");
     exit(-1);
  }
  if (results->pvalue->ndeclumps == 0) {
     if (results->pvalue->nsims == 0)
        results->pvalue->nsims = 1000;
     results->pvalue->ndeclumps = 0;
     }
  else {			/* declumping */
     results->pvalue->nsims = 10;
     if (results->pvalue->ndeclumps == 0)
        results->pvalue->ndeclumps = 300;
     }
  if (scoring->Debug) {
     (void)fprintf(io->STDOUT,
           "Using up to a score of %d\n", results->pvalue->nhist);
     }
  for (i=1; i<=results->pvalue->nsims; i++) {
     make_seq(seq1);
     if (scoring->memuse == LINEAR) { /* clear the zero row of score matrix */
        sptr = &results->S[0][1];
	for (j=1; j<=seq2->len; j++)
	   *sptr++ = 0;
     }
     srlocalS(seq1,seq2,scoring,results,io);
     if (results->bestS <= results->pvalue->nhist) {
        if (scoring->Debug) {
           (void)fprintf(io->STDOUT,
                 "best: %d of %d\n",results->bestS,results->pvalue->nhist);
           (void)fprintf(io->STDOUT,
                 "\n\nSimulation %d:\t%d\n", i, results->bestS);
           }
        results->pvalue->hist[results->bestS]++;
      }
     else {
        (void)fprintf(io->STDERR,
               "Warning: p-value score %d beyond maximum of %d\n",
		results->bestS, results->pvalue->nhist);
      }
     for (j=1; j<=results->pvalue->ndeclumps; j++) {
        trace(seq1,seq2,scoring,results,io);  /* Need alignment to declump */
        srlsdeclump(seq1,seq2,scoring,results,io);
	if (results->bestS <= results->pvalue->nhist) {
           if (scoring->Debug) {
              (void)fprintf(io->STDOUT,
                    "best: %d of %d\n",results->bestS,results->pvalue->nhist);
              }
	  results->pvalue->hist[results->bestS]++;
	}
	else {
	  (void)fprintf(io->STDERR,
                 "Warning: p-value score %d beyond maximum of %d\n",
		  results->bestS, results->pvalue->nhist);
	}
      }
   }

  getpvalue(seq1,seq2,scoring,results,io);	/* compute gamma, p and coefficient */
  /*
     Get rid of memory used for these simulated sequence lengths, and put
     everything in order to perform alignments on the real sequences.
  */
  free(seq1->code);
  free(results->S[0]);
  free(results->S);
  free(results->F);
  free(results->aligned);
  /*
     Regenerate the code from the original sequences for the pending localS.
  */
  seq1->len = save1;
  seq1->code = (char *)malloc((seq1->len + 1) * sizeof(char));
  for (i=1; i<=seq1->len; i++)  seq1->code[i] = asc2num(seq1->seq[i]);
  /*
     Initialize scoring matrix & F vector.
  */
  collen = (scoring->memuse == LINEAR) ? 2 : seq1->len + 1;
  rowlen = seq2->len + 1;
  if ((results->S=(int **)calloc(collen,sizeof(int *))) == NULL) {
     (void)fprintf(io->STDERR,
           "Error: Can't allocate scoring matrix memory.\n\n");
     exit(-1);
   }
  if ((p=(int *)calloc(collen * rowlen,sizeof(int))) == NULL) {
     (void)fprintf(io->STDERR,
            "Error: Can't allocate scoring matrix memory.\n\n");
     exit(-1);
   }
  for (i=0; i<collen; i++) {
      results->S[i] = p;
      p += rowlen;
    }
  if ((results->F=(int *)calloc(rowlen,sizeof(int))) == NULL) {
     (void)fprintf(io->STDERR,
           "Error: Can't allocate F matrix memory.\n\n");
     exit(-1);
   }
  /*
     Always allocate array for aligned pairs if self-repeats.
  */
  if ((results->aligned=(char **)calloc(collen,sizeof(char *))) == NULL) {
     (void)fprintf(io->STDERR,
           "Error: Can't allocate scoring matrix memory.\n\n");
     exit(-1);
    }
  if ((charp=(char *)calloc(collen * rowlen,sizeof(char))) == NULL) {
     (void)fprintf(io->STDERR,
           "Error: Can't allocate scoring matrix memory.\n\n");
     exit(-1);
    }
  for (i=0; i<collen; i++) {
      results->aligned[i] = charp;
      results->aligned[i][i] = 1;  /* Flag diagonals so we don't cross them */
      charp += rowlen;
    }
 
  return;
}
