/* rpt_align.c -- Prints alignment strings generated by gen_align().
                  To print alignments without using gen_align, use
                  rpt_alignment().
*/
/*
   Author: Paul Hardy, USC, February 1997.
*/

#include <stdio.h>
#include "seqaln.h"

rpt_align(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE  *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS   *results;
struct SEQALN_IO *io;
{
int i,j,k;
int chars0,chars1,chars2;   /* Characters printed out so far */
int iloc, jloc;             /* Location in current sequence  */
SEQALN_TRACE *tlist;

if (results->align_len == 0 && scoring->memuse != LINEAR) {
   (void)fprintf(io->STDERR, "Null traceback; can't print alignment.\n");
   return;
   }

if (scoring->rptscore) {
   (void)fprintf(io->STDOUT, "\nScore: %d at %s%s(%s)",
          results->bestS,
          scoring->stype == PROFILE || !seq1->reverse ?
            "" : "rev",
          scoring->stype == PROFILE || !seq1->complement ?
            "" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq1)->name :
                                        seq1->name);
   if (scoring->memuse == LINEAR) {
      (void)fprintf(io->STDOUT,"[%d] : ", results->bestSi + seq1->fstart - 1);
      (void)fprintf(io->STDOUT,"%s%s(%s)[%d]\n",
          scoring->stype == PROFILE || !seq2->reverse ?
            "" : "rev",
          scoring->stype == PROFILE || !seq2->complement ?
            "" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq2)->name :
                                        seq2->name,
                             results->bestSj + seq2->fstart - 1);
      }
   else {
      (void)fprintf(io->STDOUT,"[%d..%d] : ",
                               results->trace->i,
                            results->bestSi + seq1->fstart - 1);
      (void)fprintf(io->STDOUT,"%s%s(%s)[%d..%d]\n",
          scoring->stype == PROFILE || !seq2->reverse ?
            "" : "rev",
          scoring->stype == PROFILE || !seq2->complement ?
            "" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq2)->name :
                                        seq2->name,
                         results->trace->j,
                         results->bestSj + seq2->fstart - 1);
         }
      }
if (!scoring->rptalign || scoring->memuse == LINEAR) return;

if (scoring->width == 0) scoring->width = 60;
chars0 = chars1 = chars2 = 0;  /* No alignment letters printed yet */
iloc = results->start1;
jloc = results->start2;
tlist = results->trace;

/*
   Print the alignment.
*/
while (chars2 < results->align_len) {
   /*
      Print the top row.
   */
   /* Starting location */
   (void)fprintf(io->STDOUT,"%5d ", tlist->i == iloc ?
                 iloc + seq1->fstart - 1 : iloc + seq1->fstart);
   /* Print this line of letters */
   for (i=0; i < scoring->width && chars1 < results->align_len; i++,chars1++) {
      (void)fputc((int)results->align1[chars1], io->STDOUT);
      if (results->align1[chars1] != '-')
         iloc++;
      }
   /* Ending location */
   (void)fprintf(io->STDOUT," %5d\n", iloc + seq1->fstart - 1);
   /*
      Print the middle row.
   */
   (void)fprintf(io->STDOUT,"      ");
   /* Print matches or mismatches. */
   for (k=0; k < scoring->width && chars0 < results->align_len; k++,chars0++) {
      (void)fputc((int)results->align0[chars0], io->STDOUT);
      }
   (void)fprintf(io->STDOUT,"\n");
   /*
      Print the bottom row.
   */
   /* Starting location */
   (void)fprintf(io->STDOUT,"%5d ", tlist->j == jloc ?
                 jloc + seq2->fstart - 1 : jloc + seq2->fstart);
   /* Print this line of letters */
   for (j=0; j < scoring->width && chars2 < results->align_len; j++,chars2++) {
      (void)fputc((int)results->align2[chars2], io->STDOUT);
      if (results->align2[chars2] != '-')
         jloc++;
      }
   /* Ending location */
   (void)fprintf(io->STDOUT," %5d\n", jloc + seq2->fstart - 1);
   }
return;
}
