/* rpt_alignment.c */
/* Revised by Paul Hardy to handle profiles and place '*' between non-matches */
/*
   Author: Mark Eggert, U.S.C., 1993, revised by Paul Hardy for profiles 1995.

   Version 2.0: Modified to print conservative substitutions, Nov 1996, PH.
   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include "seqaln.h"

void rpt_alignment(seq1,seq2,scoring,results,io)
	struct SEQALN_SEQUENCE *seq1,*seq2;
	struct SEQALN_CONSTANTS *scoring;
	struct SEQALN_RESULTS   *results;
        struct SEQALN_IO *io;
{
	int ti, tj, mi, mj, bi, bj, k;
	SEQALN_TRACE *tlist = results->trace;
	SEQALN_TRACE *mlist = results->trace;
	SEQALN_TRACE *blist = results->trace;

	if (results->trace == (SEQALN_TRACE *)NULL &&
            scoring->memuse != LINEAR) {
	  (void)fprintf(io->STDERR,
                "Null traceback; can't print alignment.\n");
	  return;
	}

	if (scoring->rptscore) {
	   (void)fprintf(io->STDOUT,
                         "\nScore: %d at %s%s(%s)",
			 results->bestS,
			 scoring->stype == PROFILE || !seq1->reverse ?
				"" : "rev",
			 scoring->stype == PROFILE || !seq1->complement ?
				"" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq1)->name :
                                        seq1->name);
           if (scoring->memuse == LINEAR) {
	      (void)fprintf(io->STDOUT,"[%d] : ", results->bestSi);
              (void)fprintf(io->STDOUT,"%s%s(%s)[%d]\n",
			 scoring->stype == PROFILE || !seq2->reverse ?
				"" : "rev",
			 scoring->stype == PROFILE || !seq2->complement ?
				"" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq2)->name :
                                        seq2->name,
                             results->bestSj);
              }
           else {
	      (void)fprintf(io->STDOUT,"[%d..%d] : ",
                            results->trace->i,
                            results->bestSi);
	      (void)fprintf(io->STDOUT,"%s%s(%s)[%d..%d]\n",
			 scoring->stype == PROFILE || !seq2->reverse ?
				"" : "rev",
			 scoring->stype == PROFILE || !seq2->complement ?
				"" : "comp",
                         scoring->stype == PROFILE ?
                                        ((struct SEQALN_PROFILE *)seq2)->name :
                                        seq2->name,
                         results->trace->j,
                         results->bestSj);
	   }
        }
	if (!scoring->rptalign || scoring->memuse == LINEAR) return;

	if (scoring->width == 0) scoring->width = 60;

	if (scoring->ptype == LOCAL || (scoring->ptype == OVER && results->trace->i != 0)){
		bi=ti=mi=results->trace->i - 1;
		}
	else {
		bi=ti=mi=0;
		}
	if (scoring->ptype == LOCAL || 
	   ((scoring->ptype == FIT || scoring->ptype == OVER) && results->trace->j != 0)) {
		bj=mj=tj=results->trace->j - 1;
		}
	else {
		bj=mj=tj=0;
		}
	/*
		Print the top row, indexing through ti and tj.
	*/
	for (;;)
	{
		if (tlist->i==ti)
			(void)fprintf(io->STDOUT,"%5d ", ti);
		else
			(void)fprintf(io->STDOUT,"%5d ", ti+1);
		for (k=0; k<scoring->width;)
		{
			if (tlist->i==ti)
			{
				for (;;)
				{
					++tj;
					(void)fputc('-',io->STDOUT);
					++k;
					if (k==scoring->width)
						break;
					if (tj==tlist->j)
						break;
				}
			}
			else if (tlist->j==tj)
			{
				for (;;)
				{
					++ti;
					(void)fputc(scoring->stype==PROFILE ?
						    'X' : seq1->seq[ti],
                                                    io->STDOUT);
					++k;
					if (k==scoring->width)
						break;
					if (ti==tlist->i)
						break;
				}
			}
			else
			{
				++ti;
				(void)fputc(scoring->stype == PROFILE ?
					    seq2->seq[tj+1] : seq1->seq[ti],
                                            io->STDOUT);
				++k;
				++tj;
			}
			if (tlist->i==ti&&tlist->j==tj)
			{
				if (tlist->next==NULL)
					break;
				tlist=tlist->next;
			}
		}
		(void)fputc('\n',io->STDOUT);
		/*
			Print the middle row, indexing through mi and mj.
		*/
		if (1) {  /* reserve for future use, if '*' appears above */
		  (void)fprintf(io->STDOUT,"      ");
		  for (k=0; k<scoring->width;)
		    {
		      if (mlist->i==mi)
			{
			  for (;;)
			    {
			      ++mj;
			      (void)fputc(' ',io->STDOUT);
			      ++k;
			      if (k==scoring->width)
				break;
			      if (mj==mlist->j)
				break;
			    }
			}
		      else if (mlist->j==mj)
			{
			  for (;;)
			    {
			      ++mi;
			      (void)fputc(' ',io->STDOUT);
			      ++k;
			      if (k==scoring->width)
				break;
			      if (mi==mlist->i)
				break;
			    }
			}
		      else
			{
			  if (scoring->showmmatch == 1) {
			    if (scoring->stype != PROFILE &&
				seq1->seq[mlist->i]!=seq2->seq[mlist->j])
			      (void)fputc('*',io->STDOUT);
			    else
			      (void)fputc(' ',io->STDOUT);
			  }
			  else {
			    if (scoring->stype == PROFILE ||
			 	seq1->seq[mlist->i]==seq2->seq[mlist->j])
			      (void)fputc('|',io->STDOUT);
			    else if (scoring->mtype == SIMILARITY &&
                                     scoring->deltafn(seq1->code[mlist->i],
                                                      seq2->code[mlist->j],
                                                      scoring) > 0)
			      (void)fputc(':',io->STDOUT); /* cons'd match */
                            else
			      (void)fputc(' ',io->STDOUT);
			  }
			  ++k;
			  ++mi;
			  ++mj;
			}
		      if (mlist->i==mi&&mlist->j==mj)
			{
			  if (mlist->next==NULL)
			    break;
			  mlist=mlist->next;
			}
		    }
		}
		(void)fputc('\n',io->STDOUT);
		/*
			Print the bottom row, indexing through bi and bj.
		*/
		if (blist->j==bj)
			(void)fprintf(io->STDOUT,"%5d ", blist->j);
		else
			(void)fprintf(io->STDOUT,"%5d ", bj+1);
		for (k=0; k<scoring->width;)
		{
			if (blist->j==bj)
			{
				for (;;)
				{
					++bi;
					(void)fputc('-',io->STDOUT);
					++k;
					if (k==scoring->width)
						break;
					if (bi==blist->i)
						break;
				}
			}
			else if (blist->i==bi)
			{
				for (;;)
				{
					++bj;
					(void)fputc(seq2->seq[bj],io->STDOUT);
					++k;
					if (k==scoring->width)
						break;
					if (bj==blist->j)
						break;
				}
			}
			else
			{
				++bj;
				(void)fputc(seq2->seq[bj],io->STDOUT);
				++bi;
				++k;
			}
			if (blist->j==bj&&blist->i==bi)
			{
				if (blist->next==NULL)
					break;
				blist=blist->next;
			}
		}
		(void)fputc('\n',io->STDOUT);
		if (blist->next==NULL)
			break;
		(void)fputc('\n',io->STDOUT);
	}
return;
}
