/* rpt_row -- function to report one row of score matrix of an alignment. */
/*
   Author: Mark Eggert, U.S.C., 1993, revised by Paul Hardy.

   Version 1.2 release date: February 1996.
*/

#include <stdio.h>
#include "seqaln.h"

void rpt_row(seq1,seq2,scoring,results,io,i)
struct SEQALN_SEQUENCE  *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS   *results;
struct SEQALN_IO *io;
int i;
{
   struct SEQALN_PROFILE *pro;
   int i0,j;
   static char *chf, *inf;
   static int maxS;

   if (i == 0) {  /* First time through; print header information. */
      if (scoring->stype == PROFILE) {
         maxS = 0;
         pro = (struct SEQALN_PROFILE *)seq1;
         for (i0=0; i0<=pro->len; i0++) {
            for (j=0; j<=seq2->len; j++) {
               if (abs(results->S[i0][j]) > maxS) {
                  maxS = abs(results->S[i0][j]);
                  }
                }
             }
      }
      else if (seq1->len > seq2->len)
         maxS=abs(seq1->len * scoring->beta + scoring->alpha);
      else
         maxS=abs(seq2->len * scoring->beta + scoring->alpha);

      if (scoring->stype != PROFILE)
         seq1->seq[0]='-';
         seq2->seq[0]='-';
      if (maxS<10) {
         chf=" %2c ";
         inf=" %2d";
         }
      else if (maxS<100) {
         chf=" %3c ";
         inf=" %3d";
         }
      else if (maxS<1000) {
         chf=" %4c ";
         inf=" %4d";
         }
      else if (maxS<10000) {
         chf=" %5c ";
         inf=" %5d";
         }
      else {
         chf=" %6c ";
         inf=" %6d";
         }
      (void)fprintf(io->STDOUT,"### Scoring matrix:\n ");
      if (scoring->btype == TANDEM)
	 (void)fprintf(io->STDOUT," ");	/* so first column prints okay */
      for (j = (scoring->btype == TANDEM ? 1 : 0); j<=seq2->len; j++)
         (void)fprintf(io->STDOUT,chf, seq2->seq[j]);
      (void)fprintf(io->STDOUT,"\n");
   }
   if (i >= 0 && i <= (scoring->stype == PROFILE ? pro->len : seq1->len)) {
     if (scoring->stype == PROFILE)
       (void)fprintf(io->STDOUT," ");
     else
       (void)fprintf(io->STDOUT,"%c", seq1->seq[i]);
      if (scoring->btype == TANDEM)
	 (void)fprintf(io->STDOUT," ");	/* so first column prints okay */
     i0 = (scoring->memuse == LINEAR) ? i & 1 : i;
     for (j=0; j <= (scoring->btype==TANDEM ? seq2->len-1 : seq2->len); j++) {
       (void)fprintf(io->STDOUT,inf, results->S[i0][j]);
       /* If already used in an alignment, mark on output */
       if (results->aligned && results->aligned[i0][j] ||
	   results->linbits && results->linbits[i]) {
	 (void)fprintf(io->STDOUT,"'");
       }
       else {
	 (void)fprintf(io->STDOUT," ");
       }
     }
     (void)fprintf(io->STDOUT,"\n");
   }
   if (i == seq1->len) {	/* this is just to match version 1.00 output */
     (void)fprintf(io->STDOUT,"\n");
   }
return;
}
