/* rpt_scores -- function to report scores of an alignment. */
/*
   Author: Mark Eggert, U.S.C., 1993, revised by Paul Hardy.

   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include "seqaln.h"

void rpt_scores(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE  *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS   *results;
struct SEQALN_IO *io;
{
	struct SEQALN_PROFILE *pro;
	int i, j;
	int i0, j0;
	char *chf, *inf;
	int maxS;

	if (scoring->rptmat==1)
	{
		i0=0;
		j0=0;
	}
	else if (scoring->rptmat>1)
	{
		i0=1;
		j0=1;
	}
	else
		return;

	if (scoring->stype == PROFILE) {
           maxS = 0;
	   pro = (struct SEQALN_PROFILE *)seq1;
           for (i=0; i<=pro->len; i++) {
               for (j=0; j<=seq2->len; j++) {
                   if (abs(results->S[i][j]) > maxS) {
                      maxS = abs(results->S[i][j]);
                      }
                   }
                }
	    }
	else if (seq1->len > seq2->len)
		maxS=abs(seq1->len * scoring->beta + scoring->alpha);
	else
		maxS=abs(seq2->len * scoring->beta + scoring->alpha);

	if (scoring->stype != PROFILE)
		seq1->seq[0]='-';
	seq2->seq[0]='-';
	if (maxS<10)
	{
		chf=" %2c ";
		inf=" %2d";
	}
	else if (maxS<100)
	{
		chf=" %3c ";
		inf=" %3d";
	}
	else if (maxS<1000)
	{
		chf=" %4c ";
		inf=" %4d";
	}
	else if (maxS<10000)
	{
		chf=" %5c ";
		inf=" %5d";
	}
	else
	{
		chf=" %6c ";
		inf=" %6d";
	}
	(void)fprintf(io->STDOUT,"### Scoring matrix:\n ");
	for (j=j0; j<=seq2->len; ++j)
		(void)fprintf(io->STDOUT,chf, seq2->seq[j]);
	(void)fprintf(io->STDOUT,"\n");
	for (i=i0; i<=(scoring->stype == PROFILE ? pro->len : seq1->len); ++i)
	{
		if (scoring->stype == PROFILE)
			(void)fprintf(io->STDOUT," ");
		else
			(void)fprintf(io->STDOUT,"%c", seq1->seq[i]);
		for (j=j0; j<=seq2->len; ++j) {
			(void)fprintf(io->STDOUT,inf, results->S[i][j]);
                        /* If already used in an alignment, mark on output */
                        if (results->aligned && results->aligned[i][j]) {
                           (void)fprintf(io->STDOUT,"'");
                           }
                        else {
                           (void)fprintf(io->STDOUT," ");
                           }
                        }
		(void)fprintf(io->STDOUT,"\n");
	}
(void)fprintf(io->STDOUT,"\n");
return;
}
