/*
   topscore.c -- finds the highest score in a scoring matrix.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include <malloc.h>
#include "seqaln.h"

topscore(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
int i,j,top;
/*
   Find what the top score is for all letter combinations.
*/
if (scoring->mtype == SIMILARITY) { /* maximize score */
  switch(scoring->stype) {
  case MATRIX:
     top = gap(seq1->len + seq2->len, scoring); /* negative infinity */
     for (i=0; i<27; i++) {
       for (j=0; j<27; j++) {
         top = max(scoring->deltafn(i,j,scoring), top);
       }
     }
    break;
  case NONMATRIX:
     top = scoring->deltafn(0, 0, scoring);  /* Score to match 'A' with 'A' */
     break;
  case PROFILE:  /* Not handled. */
     break;
  }
} 
else {
  switch(scoring->stype) {
  case MATRIX:
     top = gap(seq1->len + seq2->len, scoring); /* positive infinity */
     for (i=0; i<27; i++) {
       for (j=0; j<27; j++) {
         top = max (scoring->deltafn(i,j,scoring), top);
       }
     }
    break;
  case NONMATRIX:
     top = abs(gap(seq1->len + seq2->len, scoring));
     break;
  case PROFILE: /* Not handled. */
     break;
  }
}
results->maxentry = top;

if (scoring->Debug) {
   (void)fprintf(io->STDOUT,"Top score is %d\n", top);
   }

return;
}
