/* trlsdeclump.c -- undo the effects of a tandem repeat similarity align. */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/
#include <stdio.h>
#include "seqaln.h"

trlsdeclump(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS *results;
struct SEQALN_IO *io;
{
  int i, j;
  int *left,*diag,*above,*current;
  char *pattern;

  extern int gap(),max(),max3();

  /*
     Set substitution (diagonal) flags by walking through traceback once.
  */
  markseq(seq1,seq2,scoring,results,io);

  pattern = &seq2->code[1];	/* First character of seq2->seq[] is null */

  results->bestS = results->bestSi = results->bestSj = 0;

  /* recursive S for single-letter indel weighting */
  left    = &results->S[1][0];	/* cell to the left (indel)           */
  diag    = &results->S[0][0];	/* cell up & to left (match/mismatch) */
  above   = &results->S[0][1];	/* cell above (indel)                 */
  current = &results->S[1][1];	/* starting cell                      */
  for (i=1; i<=seq1->len; i++)
    {
      *left = 0;		/* zero out first column and recompute */
      for (j=1; j<seq2->len; j++) /* Skip col. 0 the first time */
	{
	  if (results->linbits[i]) { /* this char previously aligned; skip */
	    *current = 0;
	  }
	  else {
	    *current = max3(*diag +
			    scoring->deltafn(seq1->code[i], pattern[j],
					     scoring),
			    *left + scoring->alpha,
			    *above + scoring->alpha );
	  }
	  if (*current < 0)
	    *current = 0;
	  if (scoring->Debug) {
	    (void)fprintf(io->STDOUT,
                  "S(%d,%d) = max{ (%d + %d)*%d, %d %d, %d %d, 0 } = %d\n",
			 i, j, !results->linbits[i],results->S[i-1][j-1],
			 scoring->deltafn(seq1->code[i], pattern[j],
					  scoring),
			 results->S[i][j-1], scoring->alpha,
			 results->S[i-1][j], scoring->alpha,
			 results->S[i][j] );
	    (void)fprintf(io->STDOUT,
                         "\t[%u]: [%u] <== [%u]  [%u]  [%u]\n",
			 (unsigned)&results->S[i][j], (unsigned)current,
			 (unsigned)left, (unsigned)diag, (unsigned)above);
	  }
	  left++;  diag++;  above++;  current++;
	} /* end j loop */

      /*
         Calculate very first position separately,
	 because there is no S[i-1][j-1] ==> S[0][-1]
      */
      above   -= seq2->len; /* wrap around to start of row */
      current -= seq2->len;
      if (i > 1) {
	j = 0;
      }
      else {	    /* we're at S[1][0] -- don't examine S[0][-1] */
	if (results->linbits[1]) {
	  *current = 0;
	} /* Otherwise the correct score is already there. */
	if (scoring->Debug) {
	  (void)fprintf(io->STDOUT,"S(1,0) = max{ %d*%d, 0 } = %d\n",
		       scoring->deltafn(seq1->code[i], pattern[j], scoring),
		       !results->linbits[1],
		       *current );
	  (void)fprintf(io->STDOUT,"\t[%u]: [%u] <== [%u]  [%u]  [%u]\n",
		       (unsigned)&results->S[1][0], (unsigned)current,
		       (unsigned)left, (unsigned)diag, (unsigned)above);
	}
	left++; diag++; above++; current++;
	j = 1;
      }

      for (; j<seq2->len; j++) /* Skip col. 0 the first time */
	{
	  if (j == 1) {
	    left -= seq2->len; /* wrap around in same row */
	    diag -= seq2->len;
	  }
	  if (results->linbits[i]) { /* previously aligned; skip */
	    *current = 0;
	  }
	  else {
	    *current = max3(*diag +
			    scoring->deltafn(seq1->code[i], pattern[j],
					     scoring),
			    *left + scoring->alpha,
			    *above + scoring->alpha );
	    if (*current < 0)
	      *current = 0;
	    else if (*current > results->bestS) {
	      results->bestS  = *current;
	      results->bestSi = i;
	      results->bestSj = j;
	    }
	  }
	  
	  if (scoring->Debug) {
	    (void)fprintf(io->STDOUT,
                   "S(%d,%d) = max{ (%d + %d), %d %d, %d %d, 0 } = %d\n",
		   i, j, *diag, scoring->deltafn(seq1->code[i], pattern[j],
						 scoring),
				*left, scoring->alpha,
				*above, scoring->alpha,
		   results->S[i][j] );
	    (void)fprintf(io->STDOUT,
                   "\t[%u]: [%u] <== [%u]  [%u]  [%u]\n",
		   (unsigned)&results->S[i][j], (unsigned)current,
		   (unsigned)left, (unsigned)diag, (unsigned)above);
	  }

	  left++;  diag++;  above++;  current++;
	} /* end j loop */
      j = 0;			/* j=1 first time around; j=0 thereafter */
      left++;  diag++;  above++;  current++; /* skip column zero */
    } /* end i loop */

  return;
}
