/*
  trrpt_alignment.c -- routine to print alignment from wraparound matrix.
*/
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include "seqaln.h"

/*
   Define how we detect alignment events in this scoring matrix type.
*/
#define align(i0,j0,i1,j1) (i1 == nexti(i0) && j1 == nextj(j0) &&  \
	      results->S[i1][j1] == results->S[i0][j0] +           \
	      scoring->deltafn(seq1->code[i1],seq2->code[j1+1],scoring))
#define delseq1(i0,j0,i1,j1) (i0 == i1 && \
               (j0 < j1 || (j0 == jmax && j1 == 0 && !align(i0,j0,i1,j1))))
#define delseq2(i0,j0,i1,j1) (i0  < i1 && j0 == j1)
#define previ(i0) (i0 == 0 ?    0 : i0 - 1)
#define prevj(j0) (j0 == 0 ? jmax : j0 - 1)
#define nexti(i0) (i0 + 1)
#define nextj(j0) (j0 == jmax ? 0 : j0 + 1)

/*
   tandem repeat report alignment routine.
*/
trrpt_alignment(seq1,seq2,scoring,results,io)
	struct SEQALN_SEQUENCE *seq1,*seq2;
	struct SEQALN_CONSTANTS *scoring;
	struct SEQALN_RESULTS   *results;
	struct SEQALN_IO *io;
{
int lasti, lastj;		/* previous cell coordinates in matrix      */
int j;                          /* counter for counting number of wraps     */
int k;				/* counter for looping through 4 lines      */
int jmax;			/* maximum value for j in scoring matrix    */
int jtotal;			/* real "j", in repeated pattern string     */
SEQALN_TRACE *wraptrace;        /* to count # of times we wrapped around    */
int nwraps;                     /* # of times we wrapped around             */
/*
   Allocate 4 temporary traceback lists in line[] struct:
       0 - line to print `*' above mismatches, if scoring->showmmatch=1
       1 - line to print first sequence
       2 - line to print `|' between matches, if scoring->showmmatch=0
       3 - line to print second sequence
*/
struct {
   SEQALN_TRACE *trace;		/* the traceback linked list           */
   int i,j;			/* current i and j position in matrix  */
   int togo;			/* alignment characters left on line   */
 } line[4];
  
  if (results->trace == (SEQALN_TRACE *)NULL) {
    (void)fprintf(io->STDERR,"Null traceback; can't print alignment.\n");
    return;
  }

  /* add 1 to j for wraparound since there's no column zero */
  if (scoring->rptscore) {
     (void)fprintf(io->STDOUT,
           "\nScore: %d at %s%s(%s)[%d..%d] : %d*(%s)[%d..%d]",
  		 results->bestS,
			  scoring->stype == PROFILE || !seq1->reverse ?
				"" : "rev",
			  scoring->stype == PROFILE || !seq1->complement ?
				"" : "comp",
                          scoring->stype == PROFILE ?
                                         ((struct SEQALN_PROFILE *)seq1)->name :
                                         seq1->name ,
                          results->trace->i,
                          results->bestSi,
			  scoring->window,
                          seq2->name,
                          results->trace->j+1,
                          results->bestSj+1);
       /* Count number of times we crossed over from j=jmax to j=0 */
       wraptrace = results->trace;
       nwraps=0;
       lastj = 0;
       while (wraptrace != (SEQALN_TRACE *)NULL) {
          j = wraptrace->j;
          if (j < lastj)
             nwraps++;
          wraptrace = wraptrace->next;
          lastj = j;
        }
       (void)fprintf(io->STDOUT,
             ", wrapped %d time%c\n",nwraps,nwraps==1? ' ':'s');
     }
  if (!scoring->rptalign) return;

  if (scoring->width <= 0) scoring->width = 60;
  jmax = seq2->len - 1;		/* minus 1 for wraparound matrix */
  jtotal = results->trace->j+1;	/* starting point for j in repeated pattern */

  /*
     Initialize starting points for the four lists.
  */
  for (k=0; k<=3; k++) {
     line[k].trace = results->trace;
     if (scoring->ptype == LOCAL ||
	 scoring->ptype == OVER && results->trace->i > 0) {
        line[k].i = previ(results->trace->i);
      }
     else {
        line[k].i = 0;
      }
     if (scoring->ptype == LOCAL ||
	 ((scoring->ptype == FIT || scoring->ptype == OVER) &&
	  results->trace->j > 0)) {
       line[k].j = prevj(results->trace->j);
     }
     else {
        line[k].j = 0;
      }
   }
  /*
     We either want mismatches to show above the sequences
     (scoring->showmmatch = 1), or we want matches to show
     between the sequences (scoring->showmmatch = 0), but
     not both.  Make the undesired output line have a NULL
     trace list so it won't print.
  */
  if (scoring->showmmatch) {
     line[0].trace = (SEQALN_TRACE *)NULL;
   }
  else {
     line[2].trace = (SEQALN_TRACE *)NULL;
   }
  /*
     Print alignment characters until we reach the end of the second sequence.
  */
  while (line[3].trace != (SEQALN_TRACE *)NULL) {
     /*
        Print each line in turn, skipping the nullified line.
     */
     for (k=0; k<=3; k++) {
        if (line[k].trace != (SEQALN_TRACE *)NULL) {
	   if (k == 1) { /* print position in first sequence */
	      if (line[k].trace->i==line[k].i)
	         (void)fprintf(io->STDOUT,"%8d ", line[k].i);
	      else
		 (void)fprintf(io->STDOUT,"%8d ", line[k].i+1);
	     }
	   else if (k == 3) { /* print position in second sequence */
	      /* take # of pattern chars printed mod pattern length for wrap */
	      if (!delseq2(lasti,lastj,line[k].trace->i,line[k].trace->j))
	         (void)fprintf(io->STDOUT,"%8d ", jtotal);
	      else
		 (void)fprintf(io->STDOUT,"%8d ", jtotal-1);
	     }
	   else {
	      (void)fprintf(io->STDOUT,"         "); /* skip 9 spaces */
             }
	   line[k].togo = scoring->width;
	   /*
	      Until we printed enough characters on this line,
	      or the sequence we're printing has ended.
	   */
	   while (line[k].togo > 0 && line[k].trace != (SEQALN_TRACE *)NULL) {
              /*
		 Move over one cell to the next position in the scoring matrix.
	      */
	      lasti = line[k].i;
	      lastj = line[k].j;
	      /* if we need next coords from traceback */
	      if (lasti == line[k].trace->i && lastj == line[k].trace->j) {
		 line[k].trace = line[k].trace->next;
	        }
	      /* if we haven't reached the end of alignment for this line */
	      if (line[k].trace != (SEQALN_TRACE *)NULL) {
		/* if deletion of a character from sequence one */
		 if (delseq1(lasti,lastj,line[k].trace->i,line[k].trace->j)) {
		    line[k].j = nextj(lastj);
		   }
		/* else if deletion of a character from sequence two */
		else if (delseq2(lasti,lastj,line[k].trace->i,
				 line[k].trace->j)) {
		  line[k].i = nexti(lasti);
		}
		else {			/* match or mismatch (alignment) */
		   line[k].i = nexti(lasti);
		   line[k].j = nextj(lastj);
		  }
	      }			/* finished moving over one cell */
	      if (line[k].trace != (SEQALN_TRACE *)NULL) {
		 switch(k) {
		 /*
		    Print `*' over mismatched characters if !scoring->showmmatch
		 */
		 case 0: if (seq1->seq[line[k].i] != seq2->seq[line[k].j+1] &&
			     !delseq1(lasti,lastj,line[k].i,line[k].j)      &&
			     !delseq2(lasti,lastj,line[k].i,line[k].j))     {
		            fputc('*',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         else {
			    fputc(' ',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         break;
		 /*
		    Print current letter or deletion in first sequence.
                 */
		 case 1: if (delseq1(lasti,lastj,line[k].i,line[k].j)) {
		            fputc('-',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         else {
			    fputc(seq1->seq[line[k].i],io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         break;
		 /*
		    Print `*' over mismatched characters if !scoring->showmmatch
		 */
		 case 2: if (seq1->seq[line[k].i] == seq2->seq[line[k].j+1] &&
			     !delseq1(lasti,lastj,line[k].i,line[k].j)      &&
			     !delseq2(lasti,lastj,line[k].i,line[k].j))     {
		            fputc('|',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         else {
			    fputc(' ',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         break;
		 /*
		    Print current letter or deletion in first sequence.
                 */
		 case 3: if (delseq2(lasti,lastj,line[k].i,line[k].j)) {
		            fputc('-',io->STDOUT);
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         else {
			    fputc(seq2->seq[line[k].j+1],io->STDOUT);
			    jtotal++; /* we've seen one more letter in pat. */
			    if (scoring->Debug)
			       (void)fprintf(io->STDOUT,
                                     "[%d,%d]",line[k].i,line[k].j);
			  }
		         break;
		 }		/* end case switch                         */
		 line[k].togo--;
	       }		/* end if non-null trace                   */
	    }			/* end while                               */
	}			/* end if non-null trace                   */
	if (k == 0 &&  scoring->showmmatch || k == 1 ||
	    k == 2 && !scoring->showmmatch || k == 3) {
	  fputc('\n',io->STDOUT);/* skip to next line                       */
	}
      }				/* end for k = 0 to 3                      */
     fputc('\n',io->STDOUT);	/* place blank line between alignment sets */
  }				/* end while still chars. in 2nd sequence  */

return;
}
