/* trrpt_scores -- function to report scores of a tandem repeat alignment. */
/*
   Author: Paul Hardy, U.S.C., December 1995.

   Version 1.0 release date: December 1995.
*/

#include <stdio.h>
#include "seqaln.h"


void trrpt_scores(seq1,seq2,scoring,results,io)
struct SEQALN_SEQUENCE  *seq1,*seq2;
struct SEQALN_CONSTANTS *scoring;
struct SEQALN_RESULTS   *results;
struct SEQALN_IO        *io;
{
	int i, j;
	int i0;		  /* always include column 0, since it's seq2[1] */
	char *chf, *inf;
	int minS;

	if (scoring->rptmat)
	  i0=0;
	else if (scoring->rptmat > 1)
	  i0=1;
	else
	  return;

	if (seq1->len > seq2->len)
		minS=seq1->len * scoring->alpha;
	else
		minS=seq2->len * scoring->alpha;

	seq1->seq[0]='-';
	seq2->seq[0]='-';
	if (-minS<10)
	{
		chf=" %3c";
		inf=" %2d";
	}
	else if (-minS<100)
	{
		chf=" %4c";
		inf=" %3d";
	}
	else if (-minS<1000)
	{
		chf=" %5c";
		inf=" %4d";
	}
	else if (-minS<10000)
	{
		chf=" %6c";
		inf=" %5d";
	}
	else
	{
		chf=" %7c";
		inf=" %6d";
	}
	(void)fprintf(io->STDOUT,"\n ");
	/*
           N.B.: For wraparound scoring matrices only,
	         seq1->seq[1] corresponds to results->S[1][*], and
	         seq2->seq[1] corresponds to results->S[*][0] (zero, not one).
        */
	for (j=1; j<=seq2->len; j++)
		(void)fprintf(io->STDOUT,chf, seq2->seq[j]);
	(void)fprintf(io->STDOUT,"\n");
	for (i=i0; i<=seq1->len; i++)
	{
		(void)fprintf(io->STDOUT,"%c%c", seq1->seq[i],
			     (!results->linbits || !results->linbits[i]) ?
		                 ' ' : '\'');
		for (j=0; j<seq2->len; j++) {
			(void)fprintf(io->STDOUT,
                              inf, (scoring->mtype == DISTANCE ?
                               -results->S[i][j] : results->S[i][j]));
			if (!results->linbits || !results->linbits[i])
			   fputc(' ',io->STDOUT);
			else
			   fputc('\'',io->STDOUT);
		}
		(void)fprintf(io->STDOUT,"\n");
	}
(void)fprintf(io->STDOUT,"\n");
return;
}
