/* pir_to_fasta -- program to convert PIR files to FASTA format. */
/*
   syntax:   pir_to_fasta infile outfile

   if infile is "-", input comes from stdin
   if outfile is "-", output goes to stdout

   Author: Paul Hardy, USC, November 1996
*/

#include <stdio.h>

#define MAXTITLE 512

main(argc,argv)
int argc;
char **argv;
{
int i,j,titlelen;
char inchars[MAXTITLE];
char title[MAXTITLE];
char *cptr;
FILE *pir,*fsa;

if (argc != 3) {
   (void)fprintf(stderr,"syntax:\n\n\t\t%s input-file output-file\n\n",
                 argv[0]);
   exit(-1);
   }

if (strcmp(argv[1],"-") == 0) {
   pir = stdin;
   }
else if ((pir = fopen(argv[1],"r")) == (FILE *)NULL) {
   (void)fprintf(stderr,"Error: can't open input file.\n\n");
   }

if (strcmp(argv[2],"-") == 0) {
   fsa = stdout;
   }
else if ((fsa = fopen(argv[2],"w")) == (FILE *)NULL) {
   (void)fprintf(stderr,"Error: can't open output file.\n\n");
   }

titlelen = 0;
while (fgets(inchars,MAXTITLE,pir) != NULL) {
   if (strncmp(inchars,"ENTRY",5) == 0) {  /* use for title line */
      for (i=5; isspace(inchars[i]); i++);
      for (titlelen=0;
           !isspace(inchars[i]) && titlelen < MAXTITLE;
           i++)
         title[titlelen++] = inchars[i];
      title[titlelen++] = '|';
      }
   else if (strncmp(inchars,"TITLE",5) == 0) { /* append to title line */
      for (i=5; isspace(inchars[i]); i++);
      for (;
           inchars[i] != '\n' && titlelen < MAXTITLE;
           i++)
         title[titlelen++] = inchars[i];
      title[titlelen++] = '|';
      }
   else if (strncmp(inchars,"ACCESSION",9) == 0) {
      for (i=10; isspace(inchars[i]); i++);
      for (; inchars[i] != '\n' && titlelen < MAXTITLE; i++)
         title[titlelen++] = inchars[i];
      }
   else if (strncmp(inchars,"SEQUENCE",8) == 0) { /* Read whole sequence */
      title[titlelen++] = '\0';
      (void)fprintf(fsa,">%s\n",title);
      fgets(inchars,MAXTITLE,pir);  /* Skip column number line */
      while (fgets(inchars,MAXTITLE,pir) != NULL && *inchars != '/') {
         for (cptr = inchars; *cptr; cptr++) {
            if (isalpha(*cptr)) {
               (void)fputc(*cptr,fsa);
               }
            }
         (void)fputc('\n',fsa);
         }
      }
   }

exit(0);
}
